/*
Copyright  1991-1992.  All rights reserved.
	

	You may distribute under the terms of either the GNU General Public
	License or the Artistic License, as specified in the README file.
	
	This file was developed by the FilterTop development team, specifcally 
	this file was written by Chris Priestley and Peter Lewis in early 1992
	See the Modification History for more details

FILE
	filters.c

NAME
	filters.c for ftEngine. Part of the FilterTop project

DESCRIPTION
	This source file contains code that handles the filter interface.

MODIFICATION HISTORY
	Created by Chris Priestley and Peter Lewis

	dd mmm yy	-	xxx	-	patchxx: description of patch
	25 Mar 92	-	CRP	-	Added bloated buffer handling
	25 Mar 92	-	CRP	-	Added TouchDir to properly update finder info
							(TouchDir is from comp.sys.mac.programming)
*/	


/****************************** filters.c *******************************/

/****************** includes */

#include	"filter.h"
#include	"filterInterface.h"
#include	"myString.h"
#include	"windowUtils.h"
#include	"auto-config.h"		// DNY Needed for cfgFindData callback

/******** Defines */

#define	Creator			'fENG'
#define	ftID			128;

/************* Globals */

static long					appDir;				/* dirID of application directory */
FilterInfoStruct			*filters;			/* funcHdl pointer table */
CallBacks					cb;					/* to hold callBack addresses */
extern ListHandle			gFileList, gFilterList, gPipeList, gProcessList;
extern Boolean				Done;
extern WindowPtr			gProcessWindow;

struct ActiveFilterStruct	*myActiveFilterList,
							*theActiveFilter, 
							*nextActiveFilter,
							tailActiveFilterList;	/* this one's a record, not a pointer */
long						gProcessNumber;
funcHdl						DevMarker;

extern pascal OSErr     	FilterDev(PBCallToFilter *);


/************************** FilterInit **************************/

void	FilterInit()
{
	/* make room for the array of filter handles */
	/* this should change in future to only making space for each new filter */
	
	filters = (FilterInfoStructPtr) NewPtrClear( (MAXFILTERS+1) * sizeof( FilterInfoStruct));
	if( filters == 0L)
		Done == TRUE;
	
	myActiveFilterList = 0L;
	theActiveFilter = 0L;
	nextActiveFilter = 0L;
	tailActiveFilterList.FTdata = 0L;
	gProcessNumber = 0;
	DevMarker = (funcHdl)NewHandle(0);
	 
	initCallBackArray();		/* point all the callbacks at the cb struct */
}

/************************** GetAppDir **************************/
/* get App's directory for finding the filters folder */
/* called only at startup. Used to find Filters Folder */

void			GetAppDir() 
{
	HFileInfo	myCPB;
	OSErr		myErr = noErr;
	
	myCPB.ioNamePtr = (StringPtr) NewPtrClear(40L);					
	myCPB.ioVRefNum = 0;
	myCPB.ioFRefNum = 0;
	myCPB.ioFDirIndex = -1;
	myCPB.ioNamePtr[0] = 0;
	myCPB.ioDirID = 0;
	myErr = PBGetCatInfo((CInfoPBPtr) &myCPB, FALSE);
	appDir = myCPB.ioDirID;
	DisposPtr((Ptr) myCPB.ioNamePtr);
}

/************************** AddFilters **************************/
/* add the plug-in filters to the filter List */
/* at the moment all filters in the filters folder are loaded at startup */

void			AddFilters( ListHandle listH)
{
	long		dirID;
	short		index, resNumber, i, fRef;
	HFileInfo	myCPB;
	OSErr		myErr = noErr, resErr = noErr;
	Point		cell;
	FSSpec		newFSS;
	fINFrecord	**fINFhandle;
	Boolean	targetIsFolder, wasAliased;
			
	dirID = appDir;
	myCPB.ioNamePtr = (StringPtr) NewPtrClear(40L);
	index = 1;
	resNumber = 0;
	while( myErr == noErr) 
	{
		myCPB.ioVRefNum = 0;
		myCPB.ioFDirIndex = index;
		myCPB.ioNamePtr[0] = 0;
		myCPB.ioDirID = dirID;
		myErr = PBGetCatInfo((CInfoPBPtr) &myCPB, FALSE);
		if (myErr == noErr) 
		{
			if (((myCPB.ioFlAttrib >> 4) & 0x01) == 1)	/* directory */
			{
				if( PStrCompare((char*)myCPB.ioNamePtr, (char*)"\pFilters") == TRUE) 
				{
					index = 1;
					dirID = myCPB.ioDirID;
					LDoDraw( FALSE, listH );			/* turn file list drawing off */
					while( myErr == noErr) 
					{
						myCPB.ioVRefNum = 0;
						myCPB.ioFDirIndex = index;
						myCPB.ioNamePtr[0] = 0;
						myCPB.ioDirID = dirID;
						myErr = PBGetCatInfo((CInfoPBPtr) &myCPB, FALSE);
						if( myErr == noErr) 
						{
							if(( myCPB.ioFlFndrInfo.fdCreator == Creator && myCPB.ioFlFndrInfo.fdType == 'FILT')
							|| ( myCPB.ioFlFndrInfo.fdCreator == 'fFLT' && myCPB.ioFlFndrInfo.fdType == 'APPL'))
							{
								myErr = FSMakeFSSpec( 0L, dirID, myCPB.ioNamePtr, &newFSS);
								myErr = ResolveAliasFile( &newFSS, TRUE, &targetIsFolder, &wasAliased);
								if (myErr != noErr);
								fRef = FSpOpenResFile( &newFSS, fsCurPerm);
								resErr = ResError();
								if( resErr == noErr && fRef != -1)		
								{
									fINFhandle = (fINFrecord **)GetResource( (ResType)'fINF', 128);	
									if( fINFhandle != 0L)
									{
										filters[resNumber].filterConfigH = GetResource ( 'fCFG', 128 );
										if(filters[resNumber].filterConfigH) DetachResource( filters[resNumber].filterConfigH);
										filters[resNumber].fINFrec = **fINFhandle;
										filters[resNumber].spec = newFSS;
										filters[resNumber].count = 0;
										AddLineToFilterList( listH, &newFSS);
										AddFilterIcon( listH, (short)filters[resNumber].fINFrec.IconNumber);
										resNumber += 1;
									}
									else resErr = resNotFound;
								}		
								else resErr = resNotFound;
								CloseResFile( fRef);
							}
						}
						index += 1;
					}
				}
			}
		}
		index += 1;
	}
	DisposPtr((Ptr) myCPB.ioNamePtr);
	
	// DNY Dev Filter Set up
	// This section adds the development filter to the filter list
	// The filters[*] record is the same as for other filters.
	// The identification comes from the unique name?
	
	newFSS.vRefNum = 0;
	newFSS.parID = appDir;
	FSMakeFSSpec( 0L, appDir, "\pFilterDev.Rsrc", &newFSS); 
	fRef = FSpOpenResFile( &newFSS, fsCurPerm);
	resErr = ResError();
	if( resErr == noErr && fRef != -1)
	{
		fINFhandle = (fINFrecord **)GetResource( (ResType)'fINF', 128);	
		filters[resNumber].fINFrec = **fINFhandle;
		filters[resNumber].filterConfigH = GetResource ( 'fCFG', 128 );
		if(filters[resNumber].filterConfigH) DetachResource( filters[resNumber].filterConfigH);
		pStrcpy((char *)newFSS.name,(char *)"\pDevelopment Filter");
		filters[resNumber].spec = newFSS;
		filters[resNumber].count = 0;
		filters[resNumber].filterFunc = DevMarker;
		AddLineToFilterList( listH, &newFSS);
		AddFilterIcon( listH, (short)filters[resNumber].fINFrec.IconNumber);
		CloseResFile( fRef);
	}
	DrawList( listH);
}

/********************************** AddFilterIcon *************************/
/* this is just for the filter list in the main window. */
/* probably should be in windowUtils.c */

void		AddFilterIcon( ListHandle list, short filterNum)
{
	short	bottom;
	Point	cell;
	
	bottom = (**list).dataBounds.bottom -1;
	if( bottom < 0) bottom = 0;
	SetPt( &cell, 1, bottom);
	LSetCell( &filterNum, sizeof(short), cell, list);	/* set the FSSpec */
}


/***************************** filter interface ***************************/
/***************************** filter interface ***************************/
/***************************** filter interface ***************************/


/***************************** User Handlers ***********/
/***************************** User Handlers ***********/
/***************************** User Handlers ***********/

/********************************* HandleIndividualActiveFilter ***************************/
/* user double clicked on a filter name */
/* so make the filter an active filter */
/* so set up an activeFilter record, set its state to 0 and return to main loop */
/* called from handleContentClick in FilterTop.c */
/* note this active filter will have its source and dest set to 0L */
/* therefore, it will be considered an individual active filter */
/* rather than an activeFilter that is part of a pipeLine */

void	HandleIndividualActiveFilter( short theLine )
{
		struct ActiveFilterStruct *activeFilter;
		
	if( ( activeFilter = InstallActiveFilter()) == 0L) return;	/* create new AFStruct & install at bottom of the list  */

	SetUpActiveFilterRecord( activeFilter, theLine);			/* slot it into the active filter list */
	SetupFileArray(activeFilter);								/* grab the file list in its current order. Put it in activeFilter->specs */
	InstallProcessWindowEntry( activeFilter, theLine);			/* slot it into the process window list */
}

/******************************* HandlePipeLine ******************************/
/* Grab all filters from the pipes list and create a pipeLine */
/* A pipeLine is just a set of normal activeFilter records */
/* The first activeFilter has its source set to 0L */
/* The last activeFilter has its dest set to 0L */
/* The middle activeFilters have their soure/dest set to each other */

void		HandlePipeLine()
{
	short 						i, pipeInfoLen;
	PipeInfo					pInfo;
	struct ActiveFilterStruct	*activeFilter, *lastAF;
	struct FTDataRecord			*FTData;
	Point						cell;

	lastAF = 0L;
	
	SetPt( &cell, 1, 0);
	for( i = 0; i < (**gPipeList).dataBounds.bottom; ++i)
	{
		pipeInfoLen = sizeof(PipeInfo);	/* grab info on filter from hidden cell in list */
		cell.v = i;
		LGetCell( &pInfo, &pipeInfoLen, cell, gPipeList);	/* get the FSSpec */

		if( ( activeFilter = InstallActiveFilter()) == 0L) return;	/* create new AFStruct & install at bottom of the list  */
		SetUpActiveFilterRecord( activeFilter, pInfo.filterNumber);
		if( lastAF != 0L)	/* its not the first filter in the pipeLine */
		{
			activeFilter->source = lastAF;	/* give it a souce but no data */
			lastAF->dest = activeFilter;
		}
		else SetupFileArray(activeFilter); /* must be first filter grab the file list in its current order. Put it in activeFilter->specs */
		
		lastAF = activeFilter;
		
		if( i < (**gPipeList).dataBounds.bottom - 1)	/* if it is an "inside" filter, then fake a dest */
			activeFilter->dest = activeFilter;
		InstallProcessWindowEntry( activeFilter, pInfo.filterNumber);
		activeFilter->dest = 0L;
	}
}


/********************************* ActiveFilter Record Inits ***************************/
/********************************* ActiveFilter Record Inits ***************************/
/********************************* ActiveFilter Record Inits ***************************/

/********************************* SetUpActiveFilterRecord ***************************/
/* Just fill in the general fields of an activeFilter record on init of an activeFilter */

void	SetUpActiveFilterRecord( struct ActiveFilterStruct *activeFilter, short theLine)
{

/* fill out the parts of the ActiveFilterStruct needed by filter */

	activeFilter->pb.filterShellVersion = 0x01000000;
	activeFilter->pb.cb = &cb;	
	activeFilter->pb.dlg = (DialogPtr)0L;
	activeFilter->pb.event = 0L;
	activeFilter->pb.filterConfigH = filters[theLine].filterConfigH;
	activeFilter->pb.filterVarsH = (Handle)0L;
	activeFilter->pb.streamIn = 0L;
	activeFilter->pb.streamOut = 0L;
	activeFilter->pb.var1 = 0L;
	activeFilter->pb.var2 = 0L;
	activeFilter->pb.var3 = 0L;
	activeFilter->pb.var4 = 0L;
	activeFilter->pb.var5 = 0L;

/* fill out the parts of the ActiveFilterStruct needed by shell */

	activeFilter->filterNumber = theLine;
	activeFilter->source = 0L;									/* head of pipeLine */
	activeFilter->dest = 0L;									/* tail of pipeLine */
	activeFilter->state = 0;									/* set the record as active but not yet started */
	activeFilter->FTdata = 0L;									/* set the record as active but not yet started */
	activeFilter->fINFrec = filters[theLine].fINFrec;			/* fINF for this filter */
	activeFilter->stalled = FALSE;
	activeFilter->stalledStream = 0L;
	if( filters[theLine].filterFunc != DevMarker) 			/* normal Filter */
	{
		if( filters[theLine].count == 0) 
			HandleOpenFilterRes(theLine);
	}
	
	activeFilter->filterFunc = filters[theLine].filterFunc;		/* filter code for this filter */
	
	if( filters[theLine].count == 0) 
	{
		HandleMessage(activeFilter, msgOpenFilter);
		filters[theLine].filterGlobalsH = activeFilter->pb.filterGlobalsH;
	}
	
	activeFilter->pb.filterGlobalsH = filters[theLine].filterGlobalsH;	/* filter globals for this filter */
	
	filters[theLine].count++;
}

/***************************** HandleOpenFilterRes ******************************/

OSErr	HandleOpenFilterRes( short theLine)
{
	fINFrecord	**fINFhandle;
	short		fRef;
	OSErr		resErr;
	
	resErr = noErr;

	fRef = FSpOpenResFile( &filters[theLine].spec, fsCurPerm);
	resErr = ResError();
	if( resErr == noErr && fRef != -1 && theLine < MAXFILTERS) 
	{
		filters[theLine].filterFunc = (funcHdl)GetResource(  'FILT', 128);
		resErr = ResError();
		if( filters[theLine].filterFunc != 0L ) 
		{
			DetachResource( (Handle)filters[theLine].filterFunc);
			MoveHHi( (Handle)filters[ theLine ].filterFunc);
			HLock( (Handle)filters[ theLine ].filterFunc);
		}
		CloseResFile( fRef);
	}
	return resErr;
}

/***************************** InstallActiveFilter ******************************/
/* make memory available for a new activeFilter */
/* slot the new active filter at the bottom of the linked list of active filters */

struct ActiveFilterStruct *InstallActiveFilter()
{
	struct ActiveFilterStruct *activeFilter;
	
	activeFilter = (struct ActiveFilterStruct *)AddtoTail( (listHead *)&myActiveFilterList, sizeof(struct ActiveFilterStruct) );
	return activeFilter;
}

/************************* DeleteInactiveFilter **********************************/
/* set the before, after, source and dest of surrounding active filters */
/* then delete this active filter */

OSErr	DeleteInactiveFilter(struct ActiveFilterStruct *afs)
{
	while( afs->dest != 0L && afs->dest->state == idle) 
	{
		if(afs->dest->FTdata != 0L) DeleteFTDataList(afs->dest);
		DeleteItem( (listHead *)&myActiveFilterList, (struct listEntry *)afs->dest);
	}

	if( afs->dest != 0L) afs->dest->source = 0L;	/* cut of the dest's source */
	
	DeleteProcessWindowEntry( afs);	/* knock the entry out of the process window */
	
	if(afs->FTdata) DeleteFTDataList(afs);	/* Delete Data list if any */
	
	if( filters[afs->filterNumber].filterFunc != DevMarker)	/* normal filter */
	{
		filters[afs->filterNumber].count -=1;
		if(filters[afs->filterNumber].count == 0) 
			if(filters[afs->filterNumber].filterFunc)
			{
				HandleMessage(afs, msgCloseFilter);	/* send a closeFilter msg to the filter */
				HUnlock( (Handle)filters[ afs->filterNumber ].filterFunc);
				DisposHandle( (Handle)filters[afs->filterNumber].filterFunc);
				filters[afs->filterNumber].filterFunc = 0L;
			}
	}
	else HandleMessage(afs, msgCloseFilter); /* dev filter */

	DeleteItem( (listHead *)&myActiveFilterList, (struct listEntry *)afs);	/* delete activeFilter */

	return noErr; 		
}

/********************************** SetupFileArray ***********************/
/* load the FSSpec array with the current order of the file list */
/* stuff them into an FSSpecHandle and stuff that into activeFilter->specs */
/* This routine is only used for individual activeFilters or */
/* for the head source of an activeFilter in a pipeLine */

void	SetupFileArray( struct ActiveFilterStruct *activeFilter)
{
	short					i, specLen;
	FSSpec					theSpec;
	Point					cell;
	struct FTDataRecord		*FTData;
	FInfo					fileInfo;
	OSErr					err;
	
	SetPt( &cell, 1, 0);
	for( i = 0; i < (**gFileList).dataBounds.bottom; ++i)
	{
		specLen = sizeof(FSSpec);
		cell.v = i;
		LGetCell( &theSpec, &specLen, cell, gFileList);	/* get the FSSpec */
		
		FTData = InstallNewFTDataListEntry( activeFilter);	/* attach a new FTDataRecord */
		
		FTData->fs = theSpec;	/* slot the spec */
		err = FSpGetFInfo( &theSpec, &fileInfo );		/* get the FInfo for the file */
		FTData->creator = fileInfo.fdCreator;
		FTData->type = fileInfo.fdType;
		FTData->reading = 0;
		FTData->writing = 0;
		FTData->sourceFilter = 0L;
	}
	activeFilter->state = waitingForStart;
}

/********************************* InstallNewFTDataListEntry *************************/
/* make room for a new FTDataRecord and slot it on the end of this activeFilters FTData List */

struct FTDataRecord *InstallNewFTDataListEntry( struct ActiveFilterStruct *activeFilter)
{
	struct FTDataRecord *FTData;
	
	FTData = (struct FTDataRecord *)AddtoTail( (listHead *)&activeFilter->FTdata, sizeof(struct FTDataRecord));
	return FTData;	
}

/********************************* DeleteFTDataList **********************************/
/* this routine is not called in normal circumstances */
/* normally, each FTDataRecord is deleted as the stream/file is processed */
/* so by the time an activeFilter is deleted, it should have no more FTDataRecords */

OSErr	DeleteFTDataList( struct ActiveFilterStruct *afs)
{
	struct	FTDataRecord	*FTData, *FTtemp;
	
	for( FTData = afs->FTdata; FTData != 0L; FTData = (struct FTDataRecord *)FTData->listHeader.after)
	{
		FTtemp = (struct FTDataRecord *)FTData->listHeader.after;
		if( FTData->buffer) DisposHandle(FTData->buffer);
		DisposPtr( (Ptr)FTData);
		FTData = FTtemp;
	}
	return noErr;
}

/********************************* Process Window Stuff ********************************/
/********************************* Process Window Stuff ********************************/
/********************************* Process Window Stuff ********************************/

/********************************* InstallProcessWindowEntry *****************************/

void InstallProcessWindowEntry( struct ActiveFilterStruct *activeFilter, short filterNumber)
{
	Point		cell;
	char		name[100], filterDev[] = "Development Filter";
	short		length = 100, processBottom, iconLength = sizeof(short), i;
	ProcessInfo	process;
	Rect		r;
	
	if( filters[filterNumber].filterFunc != DevMarker)	/* normal filter */
	{
		SetPt( &cell, 0, filterNumber);
		LGetCell( name, &length, cell, gFilterList);		/* get the Filter Name */
	}
	else 													/* development filter */
	{
		SetPt( &cell, 0, 0);
		length = 20;
		for( i = 0; i < length; ++i) name[i] = filterDev[i];	/* get the Filter Name */
	}
	
	processBottom = (**gProcessList).dataBounds.bottom;
	LDoDraw(false, gProcessList);
	LAddRow( 1, processBottom, gProcessList);
	cell.v = processBottom;
	LSetCell( name, length, cell, gProcessList);
	cell.h = 1;
	activeFilter->processNumber = gProcessNumber;
	process.processNumber = gProcessNumber++;
	process.boldState = 0;
	if( activeFilter->source == 0L && activeFilter->dest == 0L) process.iconNum = 1;
	else if( activeFilter->source == 0L && activeFilter->dest != 0L) process.iconNum = 2;
	else if( activeFilter->source != 0L && activeFilter->dest != 0L) process.iconNum = 3;
	else if( activeFilter->source != 0L && activeFilter->dest == 0L) process.iconNum = 4;
	LSetCell( &process, sizeof(ProcessInfo), cell, gProcessList);
	LDoDraw(true, gProcessList);
	cell.h = 0;
	LDraw( cell, gProcessList);
}

/********************************* DeleteProcessWindowEntry *****************************/

void DeleteProcessWindowEntry( struct ActiveFilterStruct *activeFilter)
{
	Point	cell;
	short		length = sizeof(ProcessInfo), i;
	ProcessInfo	process, nextProcess;
	cell.h = 1;
	for(i = 0; i < (**gProcessList).dataBounds.bottom; ++i)
	{
		length = sizeof(ProcessInfo);
		cell.v = i;
		LGetCell( &process, &length, cell, gProcessList);	/* get the ProcessInfo */
		if( activeFilter->processNumber == process.processNumber)
		{
			length = sizeof(ProcessInfo);
			cell.v +=1;
			LGetCell( &nextProcess, &length, cell, gProcessList);	/* get the ProcessInfo of next cell*/
				
			if(process.iconNum == 2)
			{
				if(nextProcess.iconNum != 4 ) nextProcess.iconNum = 2;
				else  nextProcess.iconNum = 1;
			}
			
			LSetCell( &nextProcess, length, cell, gProcessList);	/* get the ProcessInfo of next cell*/
			LDelRow( 1, i, gProcessList);
			return;
		}
	}
}

/********************************* HiliteProcessWindowEntry *****************************/

void HiliteProcessWindowEntry( struct ActiveFilterStruct *activeFilter, short hiliteState)
{
	Point			cell;
	short			length, i;
	ProcessInfo		process;
	
	cell.h = 1;	/* set the cell to the hidden column */
	for(i = 0; i < (**gProcessList).dataBounds.bottom; ++i) /* index thru cells */
	{
		length = sizeof(ProcessInfo);
		cell.v = i;
		LGetCell( &process, &length, cell, gProcessList);	/* get the ProcessInfo */
		if( activeFilter->processNumber == process.processNumber) /* check the processNumber */
		{
			process.boldState = hiliteState;	/* set the state for use by the LDEF */
			LSetCell( &process, length, cell, gProcessList);
			
			cell.h = 0; /* back to the visible collumn */
			if(hiliteState) TextFace( bold);
			LDraw( cell, gProcessList);	/* the LDEF does bolding for any updating */
			TextFace( 0);
			return;
		}
	}
}


/******************************* Process Scheduling: the loop ******************/
/******************************* Process Scheduling: the loop ******************/
/******************************* Process Scheduling: the loop ******************/

/******************************* HandleFilterLoop ******************************/
/* null event in mainLoop, so go thru filter loop */
/*also, if the state is zero, check for no source and CountFSSpec = 0 and delete it*/
/* each active filter gets a loop in sequence. */
/* so if we had active filters A, B & C, then A gets a loop, then B, then C the A again */


Boolean		HandleFilterLoop( struct ActiveFilterStruct *activeFilter)		
{
	OSErr	err;
	short	count;
	
	err = noErr;
	
	/* first, handle an active filter in either its idle, waiting or active state */
	
	if ( activeFilter->state == idle) return FALSE;								/* idle */
		
	if ( activeFilter->state == waitingForStart)								/* start */
	{	
  		err = HandleMessage(activeFilter,msgStartFiltering);					/* start it */
 		if( err != noErr && err != inProgress) ReportError( err, activeFilter);
 		if ( err == noErr)
 		{
 			err = HandleMessage(activeFilter,msgFinishFiltering);				/* finish immiediatly */
			if( err != noErr && err != inProgress) ReportError( err, activeFilter);
		}
		else if( err == inProgress) HiliteProcessWindowEntry(activeFilter, 1);	/* bold the entry in the process window */
	} 
	else if( activeFilter->state == active)											/* this one's inProgress already */
	{
		if( activeFilter->stalled)
			return FALSE;
		
		err = HandleMessage( activeFilter,msgFilterAWhile);						/* so let it filter AWhile */
		if( err != noErr && err != inProgress ) ReportError( err, activeFilter);
		if ( err != inProgress)													/* no longer inProgress */
		{	
			err = HandleMessage( activeFilter,msgFinishFiltering);				/* finish */
			if( err != noErr && err != inProgress ) ReportError( err, activeFilter);
			HiliteProcessWindowEntry(activeFilter, 0);							/* plain the entry in the process window */
		}
	}
	
	/* next, fiddle the state of the filter depending on the err code returned */

	if ( err != inProgress) 													
 	{	
 	 	if( CheckForInputAvailable( activeFilter) == noErr) activeFilter->state = waitingForStart;
		else if( activeFilter->source != 0L) activeFilter->state = idle;
		else DeleteInactiveFilter(activeFilter);
	}
	else activeFilter->state = active;												
	
	return TRUE;
}

/******************** HandleMessage ********************************/
/* filter loop calls this routine. Called whenever there is an active filter.  
The filter is passed as a parmeter. as well as the msg to send to the filter */

OSErr HandleMessage(struct ActiveFilterStruct *activeFilter, long msg)
{
	OSErr	err;
		
	activeFilter->pb.serviceRequested = msg;					/* slot the message */
	if( filters[activeFilter->filterNumber].filterFunc != DevMarker)
		err = (**activeFilter->filterFunc)( &activeFilter->pb);	/* handle already locked */
	else err = FilterDev( &activeFilter->pb);
	return err;
}

/******************************* ReportError ***************************/

void	ReportError( OSErr err, struct ActiveFilterStruct *activeFilter)
{
	Str255	param1, param2, param3, param4;
	Point	cell;
	short	length = sizeof(ProcessInfo), i, nameLength = 100;
	ProcessInfo	process;
	char	name[100];
	
	NumToString( err, param1);
	
	cell.h = 1;
	for(i = 0; i < (**gProcessList).dataBounds.bottom; ++i)
	{
		length = sizeof(ProcessInfo);
		cell.v = i;
		LGetCell( &process, &length, cell, gProcessList);	/* get the ProcessInfo */
		if( activeFilter->processNumber == process.processNumber)
		{
			cell.h = 0;
			LGetCell( name, &nameLength, cell, gProcessList);
			param2[0] = nameLength;
			for( i = 0; i < nameLength; ++i) param2[i + 1] = name[i];
			break;
		}
	}
	ParamText(param1, param2, "\p", "\p");
	NoteAlert(128, (ProcPtr) nil);
}


/**************************** Message Handlers ************************/
/**************************** Message Handlers ************************/
/**************************** Message Handlers ************************/

/**************************************** CallBacks ****************************/
/**************************************** CallBacks ****************************/
/**************************************** CallBacks ****************************/

/********************************** initCallBackArray **************************/

/* point all the callBack routines at the global CallBacks struct "cb" */
/* this struct is used by all filters to make their callBacks to the shell */

void	initCallBackArray()
{
	cb.StreamOpenIn = (ProcPtr)StreamOpenIn;
	cb.StreamCloseIn = (ProcPtr)StreamCloseIn;
	cb.GetFSSpec = (ProcPtr)GetFSSpec;
	cb.SetFSSpec = (ProcPtr)SetFSSpec;
	cb.StreamAvaibleInputBytes = (ProcPtr)StreamAvaibleInputBytes;
	cb.StreamReadBytes = (ProcPtr)StreamReadBytes;
    cb.StreamReadUpTo =  (ProcPtr)StreamReadUpTo;
	cb.StreamOpenOut = (ProcPtr)StreamOpenOut;
	cb.StreamCloseOut = (ProcPtr)StreamCloseOut;
	cb.StreamWriteBytes = (ProcPtr)StreamWriteBytes;
	cb.StreamWriteString = (ProcPtr)StreamWriteString;
	cb.cfgFindData = (ProcPtr)cfgFindData;	// DNY Added this callback
}

/************************* GetFSSpec ***************************/

pascal OSErr	GetFSSpec(FSSpec *spec, OSType *creator, OSType *type)
{
	OSErr					err;
	struct FTDataRecord		*FTData;
	
	FTData = GetFSSpecFromFTDataList( theActiveFilter);						/* get the first FTDataRecord */

	if( FTData == 0L && theActiveFilter->source == 0L) return -1;	/* no files & no source */
	if( FTData == 0L && theActiveFilter->source != 0L) return inProgress;	/* no files, but a source */
	if( FTData->writing != 0 || FTData->reading != 0 ) return inProgress;	/* its a stream */

	*spec = FTData->fs;														
	*creator = FTData->creator;	
	*type = FTData->type;	
	
	if( FTData->buffer) DisposHandle(FTData->buffer);
	DeleteItem( (listHead *)&theActiveFilter->FTdata, (struct listEntry *)FTData);	/* delete FTDataRecord */

	return noErr;
}

/************************* SetFSSpec ***************************/

pascal OSErr	SetFSSpec(short path, FSSpec *spec, OSType *creator, OSType *type)
{
	struct FTDataRecord		*FTData;
	
	if(path != 1) return -1;
	
	if(theActiveFilter->dest == 0L)	return noErr;		/* dont bother if no dest */

	FTData = InstallNewFTDataListEntry( theActiveFilter->dest);
	if(FTData == 0L) return -1;
	
	FTData->fs = *spec;
	FTData->creator = *creator;
	FTData->type = *type;
	FTData->reading = 0;
	FTData->writing = 0;
	

	if( theActiveFilter->dest->state == idle) 
		theActiveFilter->dest->state = waitingForStart;	
		
	TouchDir ( FTData->fs.vRefNum , FTData->fs.parID );

	return noErr;
}

/************************* StreamOpenIn ***************************/

pascal OSErr	StreamOpenIn(Stream *strm, FSSpec *spec, OSType *creator, OSType *type)
{
	struct FTDataRecord		*FTData;
	FInfo					fileInfo;
	OSErr					err;
	
	err = noErr;
	
	FTData = GetStreamFromFTDataList( theActiveFilter);
	
	if( FTData == 0L && theActiveFilter->source == 0L) return -1;			/* no rec, no source */
	if( FTData == 0L && theActiveFilter->source != 0L) return inProgress;	/* no rec, but a source */
	
	if( FTData->writing < 2) 	/* its either a closed file, or a file being written too */
	{
		err = FSpOpenDF( &FTData->fs, fsRdPerm, &FTData->refNum);	/* then open file */
		FTData->reading = 1;
	}
	else FTData->reading = 2;										/* its a stream */

	*strm = (Stream )FTData;
	*spec = FTData->fs;
	*creator = FTData->creator;
	*type = FTData->type;
	
	return err;
}

/************************* StreamCloseIn ***************************/

pascal OSErr StreamCloseIn(Stream *strm)
{
	struct FTDataRecord		*FTData, *FTtemp;
	struct ActiveFilterStruct *sourceFilt;
	
	FTData = ( struct FTDataRecord *)*strm;
	
	if( FTData->reading == 1) FSClose( FTData->refNum);
	if(FTData->buffer) DisposHandle(FTData->buffer);
	sourceFilt = (struct ActiveFilterStruct *)(FTData->sourceFilter);
	if(  sourceFilt != 0L              
 	&& sourceFilt->stalled                                  
 	&& (Stream)sourceFilt->stalledStream == *strm )                    
	{                                                       
		sourceFilt->stalledStream = 0L;                        
		sourceFilt->stalled = FALSE;                                           
	}                               
	
	DeleteItem( (listHead *)&theActiveFilter->FTdata, (struct listEntry *)FTData);
		
	*strm = 0L;

	return noErr;
}

/************************* StreamReadBytes ***************************/

pascal OSErr	StreamReadBytes(Stream strm, void *bufferP, long *noBytes)
{
	struct FTDataRecord		*FTData;
	OSErr					err;
	struct ActiveFilterStruct *sourceFilt;
	
	err = noErr;
	
	FTData = (struct FTDataRecord *)strm;
	
	if( FTData->reading == 1)
	{
		err = FSRead( FTData->refNum, noBytes, bufferP);
		if( err == eofErr && *noBytes > 0) return noErr;
	}
	else if( FTData->reading == 2)
	{
		GetDataFromHandle(FTData->buffer, (Ptr) bufferP, noBytes);
		sourceFilt = (struct ActiveFilterStruct *)(FTData->sourceFilter);
		if(  sourceFilt->stalled 
		&& sourceFilt->stalledStream == strm
		&& GetHandleSize( FTData->buffer) < 2048)
		{
			sourceFilt->stalledStream = 0L;
			sourceFilt->stalled = FALSE;
		}
	
		if( ( FTData->writing == 0 || FTData->writing == 3) && *noBytes < 1) /* finished. 0 means previous filter has done closeOut */
			return eofErr;
	}

	return err;
}

/************************* StreamReadUpTo ***************************/

pascal OSErr StreamReadUpTo(Stream strm, void *bufferP, long *noBytes, char upto, short *gotchar)
{
	long					i, *dummyi, *dummyhs;
	ParamBlockRec			pb;
	OSErr					oe;
	struct FTDataRecord		*FTData;
	long 					hs;
	struct ActiveFilterStruct *sourceFilt;
	
	oe = noErr;
	dummyi = &i;
	dummyhs = &hs;
	
	FTData = (struct FTDataRecord *)strm;
	
	if( FTData->reading == 1)	/* Reading a file */
	{  
		pb.ioParam.ioRefNum = FTData->refNum;
		pb.ioParam.ioBuffer = bufferP;
		pb.ioParam.ioReqCount = *noBytes;
		pb.ioParam.ioPosMode = upto << 8 | 0x80;
 		oe = PBRead( &pb, 0);
		if ( oe == eofErr && pb.ioParam.ioActCount > 0) oe = noErr;
		*noBytes = pb.ioParam.ioActCount;	
		*gotchar = ( *noBytes > 0 && ((Ptr)bufferP)[*noBytes -1] == upto);
	} 
	else /* Reading from a handle */
	{ 
		hs = GetHandleSize( FTData->buffer);

		i = 0;
		*gotchar=0;
		
		while (i<hs && !*gotchar)
		{
			*gotchar = (*(*( FTData->buffer) +i) == upto);
			i++;
		}
				
		if(i > *noBytes)
		{
			*gotchar = 0;
			i = *noBytes;
		}
		
	 	if (i == *noBytes || FTData->writing != 2 || *gotchar )
		{
            *noBytes=i;                                 
 		 	GetDataFromHandle(FTData->buffer, (Ptr) bufferP, noBytes);
			sourceFilt = (struct ActiveFilterStruct *)(FTData->sourceFilter);
			if(  sourceFilt->stalled 
			&& sourceFilt->stalledStream == strm
			&& GetHandleSize( FTData->buffer) < 2048)
			{
				sourceFilt->stalledStream = 0L;
				sourceFilt->stalled = FALSE;
			}
		}
 		else  oe=inProgress;                                    
	}
	if ( oe == noErr && *noBytes <= 0) oe = eofErr;
	
	return oe;
}


/************************* StreamAvaibleInputBytes ***************************/

pascal long		StreamAvaibleInputBytes(Stream strm)
{
	struct FTDataRecord		*FTData;
	
	FTData = (struct FTDataRecord *)strm;
	return( GetHandleSize( FTData->buffer));
}

/************************* StreamOpenOut ***************************/

pascal OSErr	StreamOpenOut(Stream *strm, short path, FSSpec *spec, OSType creator,OSType type)
{
	struct FTDataRecord		*FTData;
	OSErr					err;
	
	err = noErr;
		
	if(path != 1) return -1;

	if(theActiveFilter->dest == 0L) 						/* end of pipeLine or single filter */
		FTData = InstallNewFTDataListEntry( &tailActiveFilterList);
	else FTData = InstallNewFTDataListEntry( theActiveFilter->dest);
	
	if( FTData == 0L) return -1;
	
	*strm = (Stream )FTData;
	
	FTData->fs = *spec;
	FTData->creator = creator;
	FTData->type = type;
	FTData->reading = 0;
	FTData->writing = 0;
	
	if( theActiveFilter->dest == 0L)							/* end of pipeLine or single filter */
	{
		err = CreateOutputFile( FTData);						/* file output */
		FTData->writing = 1;
		return err;
	}
	
	if(theActiveFilter->dest->fINFrec.usesStreams == 0x00000000) /* does next filter use streams */
	{
		err = CreateOutputFile( FTData);						/* file output */
		FTData->writing = 1;
	}
	else 
	{
		FTData->buffer = NewHandle(0);							/* stream output */
		FTData->writing = 2;
		FTData->sourceFilter = (Ptr)theActiveFilter;
	
		if( theActiveFilter->dest->state == idle) 
			theActiveFilter->dest->state = waitingForStart;	
	}

	return err;
}

/************************* StreamCloseOut ***************************/

pascal OSErr	StreamCloseOut(Stream *strm)
{
	struct FTDataRecord		*FTData;
	OSErr					err;
	
	err = noErr;
	
	FTData = (struct FTDataRecord *)*strm;
	*strm = 0L;

	if(FTData->writing == 1) /* file output */
	{
		err = FSClose( FTData->refNum);	
		err = SaveTempToOriginal( FTData);
	}
	
	if( theActiveFilter->dest == 0L)
	{
		DeleteItem( (listHead *)&tailActiveFilterList.FTdata, (struct listEntry *)FTData);
		return err;
	} 
	
	if( theActiveFilter->dest->state == idle) 
		theActiveFilter->dest->state = waitingForStart;
	
	if( theActiveFilter->dest->fINFrec.usesStreams == 0x00000000)
		FTData->writing = 0;
	else FTData->writing = 3;
	
	TouchDir ( FTData->fs.vRefNum , FTData->fs.parID );
	
	return err;
}

/************************* StreamWriteBytes ***************************/

pascal OSErr	StreamWriteBytes(Stream strm, const void *bufferP, long noBytes)
{
	short 					refNum;
	OSErr					err;
	struct FTDataRecord		*FTData;
	
	err = noErr;
	
	FTData = (struct FTDataRecord *)strm;
	
	if(FTData->writing == 1) 										/* file output */
	{
		refNum = FTData->refNum;
		err = FSWrite( refNum, &noBytes, bufferP);
	}
	else if(FTData->writing == 2) 									/* stream output */
	{	
		AddDataToHandle(FTData->buffer, (Ptr)bufferP, noBytes);
		if (GetHandleSize(FTData->buffer) > 3072) 
		{                                    
			theActiveFilter->stalled = TRUE;                                         
			theActiveFilter->stalledStream = strm;                    
		}
	}
	return err;
}

/************************* StreamWriteString ***************************/

pascal OSErr	StreamWriteString(Stream strm, Str255 s)
{
	StreamWriteBytes(strm, &s[1], (long)s[0]);
}




/********************************* filter Utilities ************************/
/********************************* filter Utilities ************************/
/********************************* filter Utilities ************************/

/*********************************** GetFSSpecFromFTDataList *******************/

struct FTDataRecord *GetFSSpecFromFTDataList( struct ActiveFilterStruct *activeFilter)
{
	struct FTDataRecord *FTData;
	
	FTData = activeFilter->FTdata;
	return FTData;
}

/*********************************** GetStreamFromFTDataList *******************/

struct FTDataRecord *GetStreamFromFTDataList( struct ActiveFilterStruct *activeFilter)
{
	struct FTDataRecord *FTData;
	
	FTData = activeFilter->FTdata;
	return FTData;
}

/******************************** CheckForInputAvailable *************************/

OSErr	CheckForInputAvailable( struct ActiveFilterStruct *activeFilter)
{
	struct FTDataRecord		*FTData;
	
	FTData = activeFilter->FTdata;
	
	if( FTData == 0L) return -1;	/* theres no FTDataRec at all */
	
	if( FTData->writing > 0 ) 		/* there is one, but its a stream */
	{
		if( activeFilter->fINFrec.usesStreams == 0x00000000) return 1;	/* cant handle streams, so this Rec is unusable at the moment */
		else return noErr;	/* can handle streams, so this rec is usable */
	}
	
	if( activeFilter->FTdata != 0L) return noErr;	/* this one is, and its a FSSpec */
}

/*********************************** CreateOutputFile *********************************/

OSErr	CreateOutputFile( struct FTDataRecord *FTData)
{
	short	refNum;
	OSErr	err;
	
	err = MakeTempFile( &FTData->fs, &FTData->tempFS);	/* make a temp file */
	if( err == noErr)
	{
		err = FSpOpenDF( &FTData->tempFS, fsRdWrPerm, &refNum);	/* open temp */
		FTData->refNum = refNum;
		return err;
	}
}

/****************************** MakeTempFile ****************************/
/* this routine makes up a temp FSSpec and file in the same diretory as myFSS FSSpec */

short			MakeTempFile( FSSpec *myFSS, FSSpec *tempFSS)
{
	OSErr		err;

	*tempFSS = *myFSS;
	tempFSS->name[++tempFSS->name[0]] = '~';
	err = FSpCreate( tempFSS, 'trsh', 'trsh', -1);	/* create new file */
	if( err != noErr) 
	{
		tempFSS->name[++tempFSS->name[0]] = '~';
		err = FSpCreate( tempFSS, 'trsh', 'trsh', -1);	/* create new file */
	}
	return err;
}

/****************************** SaveTempToOriginal ****************************/
/* this routine just does a FSpExchangeFiles on the 2 FSSpecs it is given */
/* then it deletes the temp FSSpec */

short			SaveTempToOriginal( struct FTDataRecord		*FTData)
{
	FSSpec		*myFSS, *tempFSS;
	OSErr		err;
	
	myFSS = &FTData->fs;
	tempFSS = &FTData->tempFS;

	err = FSpExchangeFiles( tempFSS, myFSS);	/* swap the Finder Info */
	if( err == -43)
	{
		err = FSpCreate( myFSS, FTData->creator, FTData->type, -1);	/* create new file */
		err = FSpExchangeFiles( tempFSS, myFSS);	/* swap the Finder Info */
	}
	if( err != noErr) return err;
	err = FSpDelete( tempFSS);					/* delete the temp */
	if( err != noErr) return err;
	return noErr;
}	


/********************************** Linked List Unit ************************************/
/********************************** Linked List Unit ************************************/
/********************************** Linked List Unit ************************************/

/********************************** AddtoTail ************************************/

listPtr		AddtoTail( listHead *list, long size)
{
	struct listEntry	*item;
	
	if( *list == 0L)
	{
		*list = (struct listEntry *)NewPtrClear( size);
		(*list)->before = 0L;
		(*list)->after = 0L;
		return *list;
	}
	
	for( item = *list;  item->after != 0L; item = item->after ); /* go to bottom of list */

	item->after = (struct listEntry *)NewPtrClear( size);
	item->after->after = 0L;
	item->after->before = item;
	
	return item->after;
}


/***************************** DeleteItem **************************************/

void	DeleteItem(listHead *list, struct listEntry *item)
{
	if (*list == item) *list = item->after;
	if (item->before != 0L) item->before->after = item->after;
	if (item->after != 0L) item->after->before = item->before;
	if(item != 0L) DisposPtr( (Ptr)item);
}


/***************************** AddDataToHandle **************************************/

OSErr	AddDataToHandle(Handle h, Ptr p, long len)
{
	long size;

	size = GetHandleSize(h);
	SetHandleSize( h,size+len);
	if (MemError() != noErr) return MemError();
	BlockMove( p, *h + size, len);
	return noErr;
}

/***************************** GetDataFromHandle **************************************/

OSErr	GetDataFromHandle(Handle h, Ptr p, long *lenp)
{
	long size;

	size = GetHandleSize(h);
	if ( *lenp > size) *lenp = size;

	BlockMove( *h, p, *lenp);
	BlockMove( *h + *lenp, *h, size - *lenp);
	SetHandleSize( h, size - *lenp);
	return noErr;
}


/*************************************** TouchDir **************************/

OSErr TouchDir ( short vol , long dir )
{
	CInfoPBRec *info ;
	Str255 name ;
 	OSErr err ;
	
	info = (CInfoPBRec *)NewPtrClear( sizeof(CInfoPBRec));
	name[0] = '\0';
	info -> dirInfo . ioDrDirID = dir ;
	info -> dirInfo . ioVRefNum = vol ;
	info -> dirInfo . ioNamePtr = (StringPtr)name ;
	info -> dirInfo . ioFDirIndex = -1 ;
 
	err = PBGetCatInfo ( info , 0 ) ;
 
	if ( ! err ) {

		info -> dirInfo . ioCompletion = 0 ;
		info -> dirInfo . ioDrDirID = info -> dirInfo . ioDrParID ;
		info -> dirInfo . ioFDirIndex = 0 ;
		GetDateTime ( & ( info -> dirInfo . ioDrMdDat ) ) ;
 
		err = PBSetCatInfo ( info , 0 ) ;
	}
	DisposPtr( (Ptr)info);

	return err ;
}