/*
Copyright  1991-1992.  All rights reserved.
	

	You may distribute under the terms of either the GNU General Public
	License or the Artistic License, as specified in the README file.
	
	This file was written by Tony Jacobs in early 1992
	See the Modification History for more details

FILE
	infoManager.c

NAME
	infoManager.c for ftEngine. Part of the FilterTop project

DESCRIPTION
	This source file contains code that implements the Info Manager
	for the ftEngine application.

MODIFICATION HISTORY
	Created by Tony Jacobs

	dd mmm yy	-	xxx	-	patchxx: description of patch
	28 Mar 92	-	CRP	-	Added glue routines for ftEngine.
	
*/	

/****************************** infoManager.c *******************************/

/*********************** Includes */

#include "filter.h"				/*	these 2 files needed as they contain */
#include "filterInterface.h"	/*	typedef of FilterInfoStruct	*/
#include "zoom.h"				/*	for window zooming */
#include "stubApp.h"
#include <Packages.h>
#include "windowUtils.h"
#include "infoManager.h"

/*********************** Globals */

/* Glue globals from ftEngine */

extern ListHandle			gPipeList, gFilterList, gFileList;	/* the 3 lists */
extern	FilterInfoStruct	*filters;							/* holds spec for filters */
extern	short				LastListClicked;					/* the last list clicked */
WindowPtr					gInfoWindow;						/* the Info Manager Window */

/* These globals hold the info for the last clicked file or filter */

FSSpec						gSpec;
fINFrecord					gfINFrec;	/* fINFrecord is defined in filterInterface.h */
FInfo						gfileInfo;
HFileInfo					gHFileInfo;
ControlHandle				gDropOnable;

/********************************** HandleInfo ********************************/
/*
	This routine is called when the user clicks on an item in a list and
	chooses get info from the file menu. 
	At the moment it is filled in with an example.
	
	This is an example routine to show how to use the 2 Utilities
	It grabs the spec of the last file or filter clicked on, then
	it grabs the FInfo for that FSSpec, then it tests if the user
	clicked on a file or a filter. If a filter, it also grabs the
	filters fINFrecord. See filterInterface.h for the typedef of 
	an fINFrecord.
*/

void				HandleInfo()
{
	OSErr			err;
	
	err = GetFSSpecFromLastListClicked( &gSpec);			/* Grab the spec */
	
	err = FSpGetFInfo( &gSpec, &gfileInfo);					/* Grab the FInfo */
	
	err = GetHFileInfo( gSpec, &gHFileInfo);							/* Grab the FXInfo */

	if( LastListClicked == 1 || LastListClicked == 2)		/* Filter or Pipe List */
	{
		GetFilterInfoHandle(gSpec, &gfINFrec);				/* Grab a handle to the fINFrecord */
	}
	zoomport( gInfoWindow, 1);								/* Show the window */
	ShowWindow( gInfoWindow);
	SelectWindow( gInfoWindow);
	InvalRect( &gInfoWindow->portRect);
}

/*********************************** Window Stuff ***********************************/
/*********************************** Window Stuff ***********************************/
/*********************************** Window Stuff ***********************************/

/********************************** SetupInfoWindow ***************************/
/* 
	This routine is called at startup. It can be used to initialise the
	elements of the window. it creates at the back and dosent show it. 
	The window should be brought to the front and displayed by other
	routines. Note that it sets the graf port back to the last
	graf port on completion. Dont bother disposing of the window,
	it will just wither and die with the other windows on quiting the app.
*/

void	SetupInfoWindow()
{
	WindowPtr	oldPort;
	
	GetPort( &oldPort);
	
	gInfoWindow = GetNewWindow( 402, 0L, ( WindowPtr) 0L);
	SetPort( gInfoWindow);
	RestoreWindowPos( gInfoWindow, 131);
	SetUpDroponableButton( gInfoWindow);
	
	/* Do other initialization here */
	
	SetPort( oldPort);
}

/*********************************** ActivateInfoWindow ***********************************/
/* If becomingActive is true then the Info Manager window is being activated */
void	ActivateInfoWindow( Boolean becomingActive)
{
	if ( becomingActive )
	{
		SetPort( gInfoWindow );
		/* do activation of window components here */
	}
	else
	{
		;	/* deactivate lists, dim controls etc */
	}
}

/*********************************** UpdateInfoWindow ***********************************/
/* 
	setting port and erasing window and begin and end update are done already.
	
	The example code takes advantage of the globals gSpec, gfileInfo and, when
	appropriate the global fINFrecord
*/

void	UpdateInfoWindow()
{
	char		ost[5];
	Byte		*bp;
	short		i;
	Str255		size, date;

	TextFont(geneva);
	TextSize(9);
	MoveTo( 20, 20);					/* draw common info for file or filter */
	TextFace(1);
	DrawString( "\p Name: ");
	TextFace(0);
	DrawString( gSpec.name);
	MoveTo( 20, 40);
	TextFace(1);
	DrawString( "\p Type: ");
	TextFace(0);
	bp = (Byte *)&gfileInfo.fdType;
	for( i = 0; i < 4; ++i) ost[i] = bp[i];
	ost[5] = '\0';
	DrawString( CtoPstr( ost));
	MoveTo( 20, 60);
	TextFace(1);
	DrawString( "\p Creator: ");
	TextFace(0);
	bp = (Byte *)&gfileInfo.fdCreator;
	for( i = 0; i < 4; ++i) ost[i] = bp[i];
	ost[4] = '\0';
	DrawString( CtoPstr( ost));
	MoveTo( 20, 80);
	TextFace(1);
	DrawString( "\p Size: ");
	TextFace(0);
	NumToString( gHFileInfo.ioFlLgLen + gHFileInfo.ioFlRLgLen, size);
	DrawString( size);
	DrawString( "\p bytes");
	MoveTo( 20, 100);
	TextFace(1);
	DrawString( "\p Created: ");
	TextFace(0);
	IUDateString( gHFileInfo.ioFlCrDat , 2, date);
	DrawString( date);
	DrawChar( 32);
	IUTimeString( gHFileInfo.ioFlCrDat , 0, date);
	DrawString( date);
	MoveTo( 20, 120);
	TextFace(1);
	DrawString( "\p Modified: ");
	TextFace(0);
	IUDateString( gHFileInfo.ioFlMdDat , 2, date);
	DrawString( date);
	DrawChar( 32);
	IUTimeString( gHFileInfo.ioFlMdDat , 0, date);
	DrawString( date);
	
	
	if( gfileInfo.fdType == 'FILT'
	|| (gfileInfo.fdType == 'APPL' && gfileInfo.fdCreator == 'fFLT') )		/* its a filter, so use extra info */
	{
		if( gfileInfo.fdCreator == 'fENG') SetCtlValue( gDropOnable, 0);
		else if( gfileInfo.fdCreator == 'fFLT') SetCtlValue( gDropOnable, 1);
		MoveTo( 20, 140);
		TextFace(1);
		ShowControl( gDropOnable);
		Draw1Control( gDropOnable);
		DrawString( "\p Uses: ");
		TextFace(0);
		if( gfINFrec.usesStreams == 0x00000001) DrawString( "\pStreams");
		else DrawString( "\pFSSpecs");
		MoveTo( 20, 160);
		TextFace(1);
		DrawString( "\p Configurable: ");
		TextFace(0);
		if( gfINFrec.configPossible == 0x00000001) DrawString( "\pYes");
		else DrawString( "\pNo");
	}
	else HideControl( gDropOnable);
		
}

/*********************************** InfoContentClick ***********************************/
/* a click occured in the Info Manager Window. Here is the event record for that click */

void				InfoContentClick( EventRecord *event)
{
	Point			where, mouse;
	short			thePart, theValue;
	ControlHandle	control;
	OSErr			err;

	mouse = event->where;
	where = mouse;
	GlobalToLocal(&where);	/* local coordinates of click in Info Manager window */
	
	if ( ( thePart = FindControl( where, gInfoWindow, &control)) != 0)
	{
		if ( control == gDropOnable) 
		{
			TextFace( bold);
			if ( TrackControl( gDropOnable, where, 0))
			{
				theValue = GetCtlValue( gDropOnable);
				SetCtlValue( gDropOnable, !theValue);
				err = ConvertSingleFilterToApp( &gSpec, !theValue);
				if( theValue) 
				{
					gfileInfo.fdCreator = 'fENG';
					gfileInfo.fdType = 'FILT';
				}
				else
				{
					gfileInfo.fdCreator = 'fFLT';
					gfileInfo.fdType = 'APPL';
				}

			}
		}
	}
}

/*********************************** Utilities ***************************************/
/*********************************** Utilities ***************************************/
/*********************************** Utilities ***************************************/

/********************************** GetFSSpecFromLastListClicked **********************/
/* 
	call this routine to get the FSSpec of the file or filter
	clicked on by the user. Pass it a pointer to an FSSpec.
*/
		
OSErr				GetFSSpecFromLastListClicked( FSSpec *theSpec)
{
	short			length;
	Point			cell;
	PipeInfoRec		myPipeInfo;
	OSErr			err;
	
	err = noErr;
	SetPt( &cell, 0, 0);

	switch(LastListClicked)
 	{
 		case	1:											/* Filter List */
 			if( LGetSelect( TRUE, &cell, gFilterList))
 				*theSpec = filters[cell.v].spec;
 		break;

 		case	2:											/* PipeLine List */
 			if( LGetSelect( TRUE, &cell, gPipeList))
	 		{
				cell.h = 1;
				length = sizeof(myPipeInfo);
				LGetCell( &myPipeInfo, &length, cell, gPipeList);
				*theSpec = filters[myPipeInfo.filterNumber].spec;
			}
		break;

 		case	3:											/* File List */
  			if( LGetSelect( TRUE, &cell, gFileList))
			{
				cell.h = 1;
				length = sizeof(FSSpec);
				LGetCell( theSpec, &length, cell, gFileList);
			}
		break;
 
 		case	4:											/* Process List */
 			err = -1;
 		break;

 		default:
  			err = -1;
 		break;
 	}
 	return err;
}

/******************************* GetFilterInfoHandle ********************************/
/*
	Call this routine to get a handle to an fINFrecord of a filter.
	Pass it an FSSpec of the filter and a pointer to the global fINFrec.
*/

OSErr				GetFilterInfoHandle( FSSpec theSpec, fINFrecord	*fINF)
{
	fINFrecord		**fINFhandle;
	short			fRef;
	
	fRef = FSpOpenResFile( &theSpec, fsCurPerm);
	if( ResError()) return -1;
	fINFhandle = (fINFrecord **)GetResource( (ResType)'fINF', 128);	
	if( fINFhandle != 0L) *fINF = **fINFhandle;
	CloseResFile( fRef);
	return( noErr);
}

/************************** GetHFileInfo **************************/

OSErr			GetHFileInfo(FSSpec spec, HFileInfo *hfi) 
{
	HFileInfo	*myCPB;
	OSErr		err = noErr;
	
	myCPB = (HFileInfo *)NewPtrClear( sizeof(HFileInfo));
	myCPB->ioVRefNum = spec.vRefNum;
	myCPB->ioFRefNum = 0;
	myCPB->ioFDirIndex = 0;
	myCPB->ioNamePtr = spec.name;
	myCPB->ioDirID = spec.parID;
	err = PBGetCatInfo((CInfoPBPtr) myCPB, FALSE);
	*hfi = *myCPB;
	DisposPtr( (Ptr)myCPB);
	return err;
}

/********************* SetUpDroponableButton **************************************/

void		SetUpDroponableButton ( WindowPtr window)
{	
	Rect	r;
	
	if( gDropOnable != 0L)
		DisposeControl( gDropOnable);
		
	r = window->portRect;
	r.left += 5;
	r.right = r.left + 120;
	r.bottom -= 5;
	r.top = r.bottom - 16;
	
	gDropOnable = NewControl( window, &r, "\pDrag/Dropable",
				TRUE, 0, 0, 1, checkBoxProc | useWFont, 0L);
}

