/*
Copyright  1991-1992.  All rights reserved.
	

	You may distribute under the terms of either the GNU General Public
	License or the Artistic License, as specified in the README file.
	
	This file was developed by the FilterTop development team, specifcally 
	this file was written by Chris Priestley in early 1992
	See the Modification History for more details

FILE
	stubApp.c

NAME
	stubApp.c for ftEngine. Part of the FilterTop project

DESCRIPTION
	This source file contains code that handles saving and loading pipeLines.
	When saving a pipeLine it creates a stub application using the ftStubRes
	file in the System:Preferences folder.

MODIFICATION HISTORY
	Created by Chris Priestley

	dd mmm yy	-	xxx	-	patchxx: description of patch
	25 Mar 92	-	CRP	-	Handles saving and loading pipeLines
	
*/	

/****************************** stubApp.c *******************************/

/********************** Includes */

#include	"stubApp.h"
#include	"windowUtils.h"
#include	"filter.h"
#include	"filterInterface.h"
#include	"myString.h"
#include	"cursors.h"

/*************************** Globals */

extern ListHandle				gPipeList, gFileList;
extern FilterInfoStruct			*filters;				/* funcHdl pointer table */

short							AutoProcess;


/************************************ PipeLine Stubs *****************************/
/************************************ PipeLine Stubs *****************************/
/************************************ PipeLine Stubs *****************************/

/******************************* SavePipeLine ******************************/
/*
	Create a stub app to hold the pipeLine.
	Gather the pipeLine info and stick it in the stub app.
	Make the stub app auto process if neccessary.
*/

OSErr		SavePipeLine()
{
	short 				i, fRef, dstRef, bottom, prefVRef;
	OSErr				err;
	sFilterRec			sfRec;
	StandardFileReply	reply;
	long				length, prefDirID;
	FSSpec				stubSpec;
	fSTUBrec			**fSTUBhandle;
	FInfo				fInfo;
	
	err = noErr;
	bottom = (**gPipeList).dataBounds.bottom;	/* get the number of filters in the pipeLine */
	if( bottom == 0) return err;				/* no pipeLine */
	
	/* get the FSSpec of the ftStubRes file in the System:Preferences folder */
	
	err = FindFolder (kOnSystemDisk, kPreferencesFolderType, kDontCreateFolder, &prefVRef, &prefDirID);
		if (err != 0) return err;
	err = FSMakeFSSpec( prefVRef, prefDirID, "\pftStubRes", &stubSpec);
	
	/* Create a file to hold the stub app and the pipeLine info */
	
	AutoProcess = 0;
	PutFile( &reply);
	if (reply.sfGood)
	{
		WatchCursor();
		err = FSpDelete( &reply.sfFile);
		if( ( err = FSpCreate( &reply.sfFile, 'fSTB', 'APPL', -1)) != noErr) return err;
		if( ( err = FSpOpenDF( &reply.sfFile, fsRdWrPerm, &fRef)) != noErr) return err;
	}
	else return err;
	
	/* stuff the pipeLine info into the new file */
	
	for( i = 0; i < bottom; ++i)
	{
		GetNextPipeFilter( i, bottom, &sfRec);
		length = sizeof( sFilterRec);
		err = FSWrite( fRef, &length, &sfRec);
	}
	FSClose( fRef);
	
	/* Copy the stub app resources to the new file */
	
	if( ( err = FSpOpenRF( &stubSpec, fsRdWrPerm, &fRef)) != noErr) return err;
	if( ( err = FSpOpenRF( &reply.sfFile, fsRdWrPerm, &dstRef)) != noErr) return err;
	err = CopyResources( fRef, dstRef);	/* copy the stub code */
	err = FSClose( fRef);
	err = FSClose( dstRef);
	
	/* Make the new stub app auto process if neccessary */
	
	if(AutoProcess)
	{
		fRef = FSpOpenResFile( &reply.sfFile, fsCurPerm);
		err = ResError();
		if( err == noErr && fRef != -1)		
		{
			fSTUBhandle = (fSTUBrec **)GetResource( (ResType)'fSTB', 128);
			if( fSTUBhandle != 0L)
			{
				(**fSTUBhandle).AutoProcess = 0x00000001;
				ChangedResource((Handle)fSTUBhandle);
					if (ResError() == noErr)
						WriteResource ( (Handle)fSTUBhandle );
			}
			CloseResFile( fRef);
			
			err = FSpGetFInfo( &reply.sfFile, &fInfo);
			fInfo.fdFlags |= 0x0400;
			err = FSpSetFInfo( &reply.sfFile, &fInfo);
		}
	}
	
	TouchDir ( reply.sfFile.vRefNum , reply.sfFile.parID );	/* update finder info */

	return err;
}

/******************************* LoadPipeLine ****************************/

OSErr		LoadPipeLine( FSSpec *spec)
{
	short		i, j, fRef, totalRecs;
	sFilterRec	sfRec;
	OSErr		err;
	long		eof, length;
	fSTUBrec	**fSTUBhandle;
	
	err = noErr;
	AutoProcess = 0;
	if ((**gPipeList).dataBounds.bottom != 0)
		LDelRow( (**gPipeList).dataBounds.bottom, 0, gPipeList);
	
	fRef = FSpOpenResFile( spec, fsCurPerm);	/* open the stub app res file */
	err = ResError();
	if( err == noErr && fRef != -1)		
	{
		fSTUBhandle = (fSTUBrec **)GetResource( (ResType)'fSTB', 128);	
		if( fSTUBhandle != 0L)
		{
			if( (**fSTUBhandle).AutoProcess == 0x00000001)	/* check for auto process */
				AutoProcess = 1;
				
		}
		CloseResFile( fRef);
	}
	
	if( ( err = FSpOpenDF( spec, fsRdWrPerm, &fRef)) != noErr)
	{
		SysBeep(10);
		return err;
	}
	err = GetEOF(fRef, &eof);
	totalRecs = eof/sizeof(sFilterRec);
	for(i = 0; i < totalRecs; ++i)			/* read in each filter name of a pipeLine */
	{
		length = sizeof( sFilterRec);
		err = FSRead( fRef, &length, &sfRec);
		for( j = 0; j < MAXFILTERS; ++j)	/* compare with names of loaded filters */
		{
			if( PStrCompare( (char*)filters[j].spec.name, (char*)sfRec.spec.name) == TRUE)
			{
				HandleFilterLineSelect( j);	/* stick it in the pipe list */
				break;
			}
		}
	}
	FSClose(fRef);
	SetCursor(&arrow);
	
	if(AutoProcess)	/* if it is an auto process stub, then process the files */
	{
		if ((**gPipeList).dataBounds.bottom != 0
		&& (**gFileList).dataBounds.bottom != 0)
		{
			HandlePipeLine();
			LDelRow( (**gFileList).dataBounds.bottom, 0, gFileList); /* delete file list */
		}
	}
}

/*********************************** CopyResources ***********************/
/* make a complete copy of the ftStubRes file */

OSErr	CopyResources(short src, short dst)
{
	long	curPos, srcSize, count;
	OSErr	err;
	char	theBuf[512];
	
	err = GetEOF(src, &srcSize);
	if(srcSize ==0) return(0);
	
	while(1)
	{
		err = GetFPos( src, &curPos);
		if(curPos >= srcSize) break;	/* reached EOF */
		count = 512;
		err = FSRead(src, &count, theBuf);
		err = FSWrite(dst, &count, theBuf);
	}
	return noErr;
}

/******************************* GetNextPipeFilter ******************************/
/* get the FSSpec for the next filter in the pipeLine */

OSErr		GetNextPipeFilter(short i, short bottom, sFilterRec *sfRec)
{
	short 			pipeInfoLen;
	PipeInfo		pInfo;
	Point			cell;
	OSErr			err;
	
	err = noErr;
	
	cell.h = 1;
	pipeInfoLen = sizeof(PipeInfo);	/* grab info on filter from hidden cell in list */
	cell.v = i;
	LGetCell( &pInfo, &pipeInfoLen, cell, gPipeList);	
	
	sfRec->spec = filters[pInfo.filterNumber].spec;
	
	return err;
}


/************************************ Filter Stubs *****************************/
/************************************ Filter Stubs *****************************/
/************************************ Filter Stubs *****************************/

/******************************* ConvertSingleFilterToApp ******************************/

OSErr		ConvertSingleFilterToApp( FSSpec *spec, short which)
{
	short 				i, fRef, dstRef, prefVRef;
	OSErr				err;
	sFilterRec			sfRec;
	long				length, prefDirID;
	FSSpec				stubSpec;
	FInfo				fInfo;
	Handle				h;
	
	err = noErr;
	
	err = FSpGetFInfo( spec, &fInfo);
	if( which == 0)
	{
		dstRef = FSpOpenResFile( spec, fsRdWrPerm);
		if(ResError()) return -1;
		err = DeleteSingleFilterResources( dstRef);
		fInfo.fdCreator = 'fENG';
		fInfo.fdType = 'FILT';
		err = FSpSetFInfo( spec, &fInfo);
		TouchDir ( spec->vRefNum , spec->parID );	/* update finder info */
		CloseResFile( dstRef);
		return noErr;
	}

	/* get the FSSpec of the ftStubRes file in the System:Preferences folder */
	
	err = FindFolder (kOnSystemDisk, kPreferencesFolderType, kDontCreateFolder, &prefVRef, &prefDirID);
		if (err != 0) return err;
	err = FSMakeFSSpec( prefVRef, prefDirID, "\pftStubRes", &stubSpec);
	
	/* Copy the stub app resources to the filter */
	
	fRef = FSpOpenResFile( &stubSpec, fsCurPerm);
	if( ResError()) return -1;		
	dstRef = FSpOpenResFile( spec, fsRdWrPerm);
	if(ResError())		
	{
		CloseResFile( fRef);
		return -1;
	}
	err = CopySingleFilterResources( fRef, dstRef);	/* copy the stub code */
	CloseResFile( fRef);
	CloseResFile( dstRef);
	
	fInfo.fdCreator = 'fFLT';
	fInfo.fdType = 'APPL';
	err = FSpSetFInfo( spec, &fInfo);

	TouchDir ( spec->vRefNum , spec->parID );	/* update finder info */

	return err;
}

/******************************* CopySingleFilterResources **********************/
/* 
	Grab all neccessary resources from ftStubRes in the 
	System Folder:Preferences Folder and stick them in the filter 
*/

OSErr		CopySingleFilterResources( short srcRef, short dstRef)
{
	OSErr	err;
	
	UseResFile( dstRef);
	if( Count1Resources( 'BNDL')) return -1;	/* already there */
	
	if(( err = AddSingleResource( srcRef, dstRef, 'BNDL', 128)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'CODE', 0)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'CODE', 1)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'CODE', 2)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'DATA', 0)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'DREL', 0)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'FREF', 128)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'FREF', 129)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'FREF', 130)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'SIZE', -1)) != noErr) return -1;
	if(( err = AddSingleResource( srcRef, dstRef, 'ZERO', 0)) != noErr) return -1;
		
	return noErr;
}

/******************************* AddSingleResource ******************************/

OSErr	AddSingleResource( short srcRef, short dstRef, ResType rType, short rID)
{
	Handle	h;
	
	UseResFile( srcRef);
	h = GetResource( rType, rID);
	if(!h) return -1;
	DetachResource(h);
	UseResFile( dstRef);
	AddResource( h, rType, rID, "\p");
	if( ResError())
	{
		DisposHandle(h);
		return -1;
	}
	WriteResource( h);
	return noErr;	
}

/******************************* DeleteSingleFilterResources **********************/

OSErr		DeleteSingleFilterResources( short fRef)
{
	OSErr	err;
	
	UseResFile( fRef);
	if( !Count1Resources( 'BNDL')) return -1;	/* No such resource */
	
	if(( err = DeleteSingleResource( fRef, 'BNDL', 128)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'CODE', 0)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'CODE', 1)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'CODE', 2)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'DATA', 0)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'DREL', 0)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'FREF', 128)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'FREF', 129)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'FREF', 130)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'SIZE', -1)) != noErr) return -1;
	if(( err = DeleteSingleResource( fRef, 'ZERO', 0)) != noErr) return -1;
		
	return noErr;
}

/******************************* DeleteSingleResource ******************************/

OSErr	DeleteSingleResource( short fRef, ResType rType, short rID)
{
	Handle	h;
	
	UseResFile( fRef);
	h = GetResource( rType, rID);
	if(!h) return -1;
	RmveResource( h);
	if( !ResError()) WriteResource( h);
	DisposHandle(h);
	return noErr;	
}

/******************************* LoadSingleFilter ****************************/

OSErr		LoadSingleFilter( FSSpec *spec)
{
	short		i;
	OSErr		err;
	
	err = noErr;

	if ((**gPipeList).dataBounds.bottom != 0)
		LDelRow( (**gPipeList).dataBounds.bottom, 0, gPipeList);
	
	for( i = 0; i < MAXFILTERS; ++i)	/* compare with names of loaded filters */
	{
		if( PStrCompare( (char*)filters[i].spec.name, (char*)spec->name) == TRUE)
		{
			HandleFilterLineSelect( i);	/* stick it in the pipe list */
			break;
		}
	}

	SetCursor(&arrow);
	
	if ((**gPipeList).dataBounds.bottom != 0
	&& (**gFileList).dataBounds.bottom != 0)
	{
		HandlePipeLine();
		LDelRow( (**gFileList).dataBounds.bottom, 0, gFileList); /* delete file list */
	}
	return err;
}



/************************* CustomPutFile Dialog ************************/
/************************* CustomPutFile Dialog ************************/
/************************* CustomPutFile Dialog ************************/

/**************************** PutFile *************************/
/* 
	Custom PutFile dialog so we can have an Auto Process check box
	If auto process is checked, it the MyStubDlg proc sets a global.
	That global is used to decide on the type of stub app that is
	created when a pipeLine is saved.
	An auto process stub app will cause files drag dropped to it to
	load and process automatically. A normal stub app just loads any
	files drag dropped to it.
*/

void				PutFile(StandardFileReply *reply)
{
	Point		where;
	
	reply->sfGood = FALSE;
	SetPt(&where, -1, -1);
	
	CustomPutFile(	"\p",						/* Prompt string */
					"\p",						/* Default File Name */
					reply,						/* StandardFileReply Ptr */
					500,						/* Resource number for Custom Dialog Box */
					where,						/* Dialog Position */
					(ProcPtr)MyStubDlg,			/* Custom Dialog Box procedure */
					0L,							/* filter proc */
					0L,							/* active list */
					0L,							/* activate proc */
					0L );						/* data pointer */
	}

/********************** MyStubDlg ******************************/

pascal short	MyStubDlg( short item, DialogPtr theDialog, Ptr myDataPtr )
{
	short			itemType;
	Rect			itemRect;
	ControlHandle	itemHandle;

	switch( item ) {
		case -1:		/* startup */
				
				GetDItem( theDialog, 13, &itemType, (Handle*)&itemHandle, &itemRect );
				SetCtlValue( itemHandle, 0 );
				return item;
				break;

		case 13:	/* the auto process check box */
			GetDItem( theDialog, 13, &itemType, (Handle*)&itemHandle, &itemRect );
			if(GetCtlValue( itemHandle)) 
			{
				SetCtlValue( itemHandle, 0 );
				AutoProcess = 0;				/* set global */
			}
			else
			{
				SetCtlValue( itemHandle, 1 );
				AutoProcess = 1;				/* set global */
			}
			return sfHookNullEvent;
			break;

		default:			/* all the rest handled in normal way */
			return item;
			break;
	}
}
