/*
Copyright  1991-1992.  All rights reserved.
	

	You may distribute under the terms of either the GNU General Public
	License or the Artistic License, as specified in the README file.
	
	This file was developed by the FilterTop development team, specifcally 
	this file was written by Chris Priestley in early 1992
	See the Modification History for more details

FILE
	windowUtils.c

NAME
	windowUtils.c for ftEngine. Part of the FilterTop project

DESCRIPTION
	This source file contains code provides the UI for ftEngine.
	It contains the code for all the windows and the lists.

MODIFICATION HISTORY
	Created by Chris Priestley

	dd mmm yy	-	xxx	-	patchxx: description of patch
	25 Mar 92	-	CRP	-	No Changes
	
*/	

/****************************** windowUtils.c *******************************/

/************************ Includes */

#include		"windowUtils.h"
#include 		"cursors.h"
#include 		"zoom.h"
#include 		"prefs.h"

/****************** Globals ********/

extern WindowPtr	ftWindow, gInfoWindow;
WindowPtr			gFilesWindow, gProcessWindow;
extern WindowPtr	ftWindow;
ListHandle			gFileList, gFilterList, gPipeList, gProcessList;
Rect				gFileListRect, gFilterListRect, gPipeListRect, gProcessListRect;
ControlHandle		gDeleteButton, gDeleteFiltersButton, gPipesButton;

/************************************* FileWindowSetup *********************/

void		FileWindowSetup()
{
	Rect	dataBounds;
	Point	cellSize, where, thePoint;
	Str255	files = "\pFiles";
	GrafPtr	oPort;
	
	GetPort( &oPort);

	gFilesWindow = GetNewWindow( 400, 0L, ( WindowPtr) -1L);
	SetWTitle( gFilesWindow, files);
	SetPort( gFilesWindow);
	TextFont( 0);
	TextSize( 12);
	RestoreWindowPos( gFilesWindow, 128);

	SetUpListRect( gFilesWindow, &gFileListRect);
	SetRect( &dataBounds, 0, 0, 2, 0);
	SetPt( &cellSize, 530, 18);
	gFileList = LNew( &gFileListRect, &dataBounds, cellSize, 128, 
					gFilesWindow, true, true, false, true);
	( **gFileList).lClikLoop = ( ProcPtr)myLClikLoop;
	SetUpDeleteButton( gFilesWindow);
	
	
	zoomport( gFilesWindow, 1);
	ShowWindow( gFilesWindow);
}

/************************** RestoreWindowPos ************************/

void RestoreWindowPos( WindowPtr window, short rID)
{
	Rect r;
	Point p;
	
	p.v = window->portRect.top;
	p.h = window->portRect.left;
	r = window->portRect;
	LocalToGlobal( &p);
	GetWindowPref( &r, &p, rID);
	SizeWindow( window, r.right, r.bottom, FALSE);
	MoveWindow( window, p.h, p.v, FALSE);
}

/************************** SaveWindowPos ************************/

void SaveWindowPos( WindowPtr window, short rID)
{
	Rect r;
	Point p;
	GrafPtr	oldPort;
	
	GetPort( &oldPort);
	SetPort( window);
	
	p.v = window->portRect.top;
	p.h = window->portRect.left;
	r = window->portRect;
	LocalToGlobal( &p);
	SetWindowPref( &r, &p, rID);
	
	SetPort(oldPort);
}

/***************************** SaveWindowPositions *********************/

void	SaveWindowPositions()
{
	SaveWindowPos( gFilesWindow, 128);
	SaveWindowPos( gProcessWindow, 129);
	SaveWindowPos( ftWindow, 130);
	SaveWindowPos( gInfoWindow, 131);
}

/************************************* ProcessWindowSetup *********************/

void		ProcessWindowSetup()
{
	Rect	dataBounds;
	Point	cellSize, where;
	Str255	process = "\pProcesses";
	GrafPtr	oPort;
	
	GetPort( &oPort);

	gProcessWindow = GetNewWindow( 401, 0L, ( WindowPtr) -1L);
	SetWTitle( gProcessWindow, process);
	SetPort( gProcessWindow);
	TextFont( 0);
	TextSize( 12);
	RestoreWindowPos( gProcessWindow, 129);

	SetUpListRect( gProcessWindow, &gProcessListRect);
	SetRect( &dataBounds, 0, 0, 2, 0);
	SetPt( &cellSize, 530, 18);
	gProcessList = LNew( &gProcessListRect, &dataBounds, cellSize, 131, 
					gProcessWindow, true, true, false, true);
	
	zoomport( gProcessWindow, 1);
	ShowWindow( gProcessWindow);
}

/************************************* FilterListSetup *********************/

void		FilterListSetup()
{
	Rect	dataBounds;
	Point	cellSize, where;
	GrafPtr	oPort;
	
	GetPort( &oPort);

	SetPort( ftWindow);
	TextFont( 0);
	TextSize( 12);

	SetUpFilterButtons( ftWindow);
	SetUpListRect( ftWindow, &gFilterListRect);
	SetUpListRect( ftWindow, &gPipeListRect);
	AdjustFilterRects();
	SetRect( &dataBounds, 0, 0, 2, 0);
	SetPt( &cellSize, 530, 18);
	gFilterList = LNew( &gFilterListRect, &dataBounds, cellSize, 130, 
					ftWindow, true, true, false, true);
	( **gFilterList).lClikLoop = ( ProcPtr)myLClikLoop;
	gPipeList = LNew(&gPipeListRect, &dataBounds, cellSize, 129,
					ftWindow, true, true, false, true);
	( **gPipeList).lClikLoop = (ProcPtr)myLClikLoop;
	
	SetPort( oPort);
}

/**************************** AdjustFilterRects **************************/

void	AdjustFilterRects()
{
	gFilterListRect.right -= (ftWindow->portRect.right) / 2;
	gPipeListRect.left = gFilterListRect.right +16;
}

/********************* SetUpFilterButtons **************************************/

void		SetUpFilterButtons ( WindowPtr window )
{	
	Rect	r;
	Str255	Clear = "\pClear", Pipe = "\pPipe";
	
	if( gDeleteFiltersButton != 0L)
		DisposeControl( gDeleteFiltersButton);
	if( gPipesButton != 0L)
		DisposeControl( gPipesButton);
		
	r = window->portRect;
	r.right = r.right -14;
	r.top = r.bottom - 15;
	r.left = r.right - 65;
	r.bottom += 1;
	
	gPipesButton = NewControl( window, &r, Pipe,
				TRUE, 0, 0, 0, squareButProc, 0L );	
				
	r.right = r.left +1;
	r.left = r.right - 65;
	
	gDeleteFiltersButton = NewControl( window, &r, Clear,
				TRUE, 0, 0, 0, squareButProc, 0L );	
}

/********************* SetUpDeleteButton **************************************/

void		SetUpDeleteButton ( WindowPtr window)
{	
	Rect	r;
	Str255	Clear = "\pClear";
	
	if( gDeleteButton != 0L)
		DisposeControl( gDeleteButton);
		
	r = window->portRect;
	r.top = r.bottom - 15;
	r.left -= 1;
	r.right = r.left + 70;
	r.bottom += 1;
	
	gDeleteButton = NewControl( window, &r, Clear,
				TRUE, 0, 0, 0, squareButProc, 0L);		/* see the new CDEF */
}

/***************************** AdjustWindowItems **************************/

void		AdjustWindowItems( WindowPtr window)
{
	if ( window == gFilesWindow)
	{
		SetUpListRect( window, &gFileListRect);
		LSize( gFileListRect.right, gFileListRect.bottom, gFileList);
	}
	else if ( window == gProcessWindow)
	{
		SetUpListRect( window, &gProcessListRect);
		LSize( gProcessListRect.right, gProcessListRect.bottom, gProcessList);
	}
	else if( window == ftWindow)
	{
		SetUpListRect( window, &gFilterListRect);
		SetUpListRect( window, &gPipeListRect);
		AdjustFilterRects();
		LSize( gFilterListRect.right, gFilterListRect.bottom, gFilterList );
		LSize( gPipeListRect.right - gPipeListRect.left, gPipeListRect.bottom, gPipeList );
	}
	InvalRect( &window->portRect);
}

/********************* SetUpListRect **************************************/

void		SetUpListRect ( WindowPtr window, Rect *list)
 {	
	*list = window->portRect;
	list->right -= 15;				/* leave room for scroll bars */
 	list->bottom -= 15;
}

/*************************************** AddLineToList ************************/
/* make a new row at the bottom of the list and insert the label and FSSpec */
/* the FSSpec is experimental. Presumably any struct or a pointer could be put in */
/* attaching a structure allows for easy shuffling of the list later on */
/* large structures however would limit the number of rows in the list because 32k barrier */
/* if large number of rows anticipated, better to insert a pointer to a struct */
/* but inserting a pointer means extra code to handle the structs outside the List Manager */

void		AddLineToList( ListHandle list, FSSpec *theFSSpec)
{
	Point	labelCell, specCell;
	
	SetPt( &labelCell, 0, ( **list).dataBounds.bottom);			
	SetPt( &specCell, 1, ( **list).dataBounds.bottom);			
	LAddRow( 1, ( **list).dataBounds.bottom, list);				/* new cell at bottom */
	LSetCell( &theFSSpec->name[1], theFSSpec->name[0], labelCell, list);	/* insert string in new cell */
	LSetCell( theFSSpec, sizeof(FSSpec), specCell, list);		/* insert FSSpec in new cell */
}

/*************************************** AddLineToFilterList ************************/

void		AddLineToFilterList( ListHandle list, FSSpec *theFSSpec)
{
	Point	labelCell;
	
	SetPt( &labelCell, 0, ( **list).dataBounds.bottom);			
	LAddRow( 1, ( **list).dataBounds.bottom, list);				/* new cell at bottom */
	LSetCell( &theFSSpec->name[1], theFSSpec->name[0], labelCell, list);	/* insert string in new cell */
}

/************************* DrawList *************************/

void		DrawList( ListHandle list)
{	
	short	i;
	Point	cell;
						
	LDoDraw( TRUE, list);								/* turn list drawing on */
	for ( i = 0; i < ( **list).dataBounds.bottom; ++i)	/* draw each new line in list */
	{
		SetPt( &cell, 0,  i);
		LDraw( cell, list);
	}
}


/******************************** myLClikLoop ********************************/
/* callback routine from LClick for lists: stuff address into ListRec.lClickLoop */
/* see IM IV 266 */
/* this routine does nothing but loop thru twice and then lie to the List Manager that the mouse is up */
/* it loops thru twice so the List Manager will select the first cell clicked the first loop */
/* it exits on the second loop to make LClick give control of the mouse to the custom routine HandleListDrag */
	
short			gFirstClick = 1;
	
pascal Boolean	myLClikLoop()
{
	if( !StillDown( ))
	{
		gFirstClick = 1;
		return FALSE;
	}
	if ( gFirstClick)
	{
		gFirstClick = 0;	/* stay in clikLoop first time thru so List Manager will select first cell clicked on */
		return TRUE;
	}
	else
	{
		gFirstClick = 1;	/* Break out of clickLoop even though mouse still down */
		return FALSE;		/* IM says you should always return TRUE, but we want control */
	}
}

/******************************** HandleFilterListDrag ********************************/
/* Routine called after myLClikLoop has cheekily returned false while the mouse still down */

void			HandleFilterListDrag( ListHandle list)
{
	Point		mouse;
	Rect		newRect, oldRect;
	Cell		dstCell;
	RgnHandle	theRgn;
	
	if( ( **list).dataBounds.bottom == 0) return;		/* only interested if something in list */
	GetMouse( &mouse);
	oldRect = ( **list).rView;
	if( !PtInRect( mouse, &oldRect)) return;			/* only interested if in rView */
	
	if( !StillDown( )) return;							/* only interested if mouse still down */
	else
	{
		dstCell.h = 0;									
		dstCell.v = 0;
		while( dstCell.v != ( **list).visible.top) dstCell.v += 1;	/* wind thru to first visible cell */
		dstCell.v += mouse.v / ( **list).cellSize.v;				/* add the vertical mouse position */

		if( dstCell.v > ( **list).dataBounds.bottom - 1)	/* keep it within bounds of active cells */
			return;
		else
			LRect( &newRect, dstCell, list);
		newRect.right = ( **list).rView.right;		/* now react to mouse position */
			
		oldRect.right *= 2;
		oldRect.right += 16;
		theRgn = NewRgn();
		RectRgn( theRgn, &newRect);
		DragGrayRgn( theRgn, mouse, &oldRect, &oldRect, 0, 0L);
		DisposeRgn( theRgn);
		GetMouse(&mouse);
		oldRect.left = (oldRect.right / 2) + 16;
		if( PtInRect( mouse, &oldRect))
			HandleFilterLineSelect(dstCell.v);
	}
}

/***************************** HandleFilterLineSelect *********************/

void		HandleFilterLineSelect( short theLine)
{
	short		pipeListBottom, i, length = 63, pipeInfoSize = sizeof(PipeInfo), shortSize = sizeof( short);
	Point		cell, hiddenCell;
	char		line[63];
	PipeInfo	myPipeInfo;
	Rect		cellRect;
	
	myPipeInfo.filterNumber = theLine;
	SetPt( &cell, 0, theLine);						/* insert line in list */
	LGetCell( &line[1], &length, cell, gFilterList);
	line[0] = length;
	LDoDraw( FALSE, gPipeList);
	cell.h = 1;
	LGetCell( &myPipeInfo.filterType, &shortSize, cell, gFilterList);
	AddLineToPipeList( gPipeList, (StringPtr)line);
	SetPt( &hiddenCell, 1, ( **gPipeList).dataBounds.bottom -1);			
	LSetCell( &myPipeInfo, sizeof(myPipeInfo), hiddenCell, gPipeList);		/* insert short in new cell */
	hiddenCell.h = 0;
	LDoDraw( TRUE, gPipeList);
	LDraw( hiddenCell, gPipeList);
}

/*************************************** AddLineToPipeList ************************/

void		AddLineToPipeList(ListHandle list, Str255 string)
{
	Point	cell;
	char	line[63];
	short	i;
	
	SetPt( &cell, 0, (**list).dataBounds.bottom );			/* insert line in list */
	LAddRow( 1, (**list).dataBounds.bottom, list );
	for ( i = 0; i < string[0]; ++i) line[i] = string[i + 1];
	LSetCell( line, string[0], cell, list );
}


/******************************** HandleListDrag ********************************/
/* Routine called after myLClikLoop has cheekily returned false while the mouse still down */

void			HandleListDrag( ListHandle list)
{
	Point		mouse;
	Rect		newRect, oldRect;
	Cell		dstCell, srcCell;
	
	if( ( **list).dataBounds.bottom == 0) return;		/* only interested if something in list */
	GetMouse( &mouse);
	oldRect = ( **list).rView;
	if( !PtInRect( mouse, &oldRect)) return;			/* only interested if in rView */
	oldRect.top = oldRect.bottom;						/* so it wont be drawn first time thru loop */
	
	if( !StillDown( )) return;							/* only interested if mouse still down */
	else
	{
		while( WaitMouseUp( ))
		{
			GetMouse( &mouse);							/* Set up newRect. Accomodate scroll state */
			dstCell.h = 0;									
			dstCell.v = 0;
			while( dstCell.v != ( **list).visible.top) dstCell.v += 1;	/* wind thru to first visible cell */
			dstCell.v += mouse.v / ( **list).cellSize.v;				/* add the vertical mouse position */

			if( dstCell.v > ( **list).dataBounds.bottom - 1)	/* keep it within bounds of active cells */
			{
				dstCell.v = ( **list).dataBounds.bottom - 1;
				LRect( &newRect, dstCell, list);
			}
			else
				LRect( &newRect, dstCell, list);
			newRect.right = ( **list).rView.right;		/* now react to mouse position */
			
			if( mouse.v < ( **list).rView.top)			/* scroll down */
			{
				FrameRect( &oldRect);					/* delete old gray rect before scroll */
				LScroll( 0, -1, list);
				oldRect.top = oldRect.bottom;			/* make sure old wont be drawn again */
			}
			else if( mouse.v > ( **list).rView.bottom)	/* scroll up */
			{
				FrameRect( &oldRect);					/* delete old gray rect before scroll */
				LScroll( 0, 1, list);
				oldRect.top = oldRect.bottom;			/* make sure old wont be drawn again */
			}
			else if( newRect.top != oldRect.top)		/* not still in the same cell */
			{
				PenPat( gray);							/* OK, OK, we'll do a gray rect drag */
				PenMode( notPatXor);
				FrameRect( &oldRect);					/* delete old gray rect */
				FrameRect( &newRect);					/* draw new gray rect */
				oldRect = newRect;						/* stuff new into old for next loop */
			}
		}
	}
	FrameRect( &oldRect);								/* delete last gray rect */
	PenNormal( );										/* all drawing done before the shuffle */
	srcCell = LLastClick( list);						/* shuffle the cells */

	if( list == gPipeList)
	{
		if( srcCell.v < ( **list).dataBounds.bottom) 
			ShufflePipeList( srcCell, dstCell, list);
	}
	else if( list == gFileList)
	{
		if( srcCell.v < ( **list).dataBounds.bottom) 
			ShuffleFileList( srcCell, dstCell, list);
	}
}

/******************************* ShufflePipeList ********************************/
/* move the source row to the position of the destination row */
/* put the label in column 0 for display and hide the FSSpec in column 1 */

void		ShufflePipeList( Cell srcRow, Cell dstRow, ListHandle list)
{
	Ptr			dataPtr;
	short		dataLen, pipeInfoSize = sizeof(PipeInfo), i;
	PipeInfo	myPipeInfo;
	Rect		cellRect;
	
	if ( dstRow.v != srcRow.v)						/* not the same row */
	{
		WatchCursor();
		dataLen = 100;
		dataPtr = NewPtrClear( 100L);
		LGetCell( dataPtr, &dataLen, srcRow, list);	/* get the label */
		srcRow.h = 1;
		LGetCell( &myPipeInfo, &pipeInfoSize, srcRow, list);	/* get the hidden short */
		LDelRow( 1, srcRow.v, list);
		LDoDraw( FALSE, list);
		LAddRow( 1, dstRow.v, list);
		LSetCell( dataPtr, dataLen, dstRow, list);	/* set the label */
		dstRow.h = 1;
		LSetCell( &myPipeInfo, pipeInfoSize, dstRow, list);	/* set the hidden short */
		DrawList( list);
		DisposPtr( dataPtr);
		ArrowCursor();
	}
}

/******************************* ShuffleFileList ********************************/
/* move the source row to the position of the destination row */
/* put the label in column 0 for display and hide the FSSpec in column 1 */

void		ShuffleFileList( Cell srcRow, Cell dstRow, ListHandle list)
{
	Ptr		dataPtr;
	short	dataLen, specLen = sizeof(FSSpec);
	FSSpec	theSpec;
	
	if ( dstRow.v != srcRow.v)						/* not the same row */
	{
		WatchCursor();
		dataLen = 100;
		dataPtr = NewPtrClear( 100L);
		LGetCell( dataPtr, &dataLen, srcRow, list);	/* get the label */
		srcRow.h = 1;
		LGetCell( &theSpec, &specLen, srcRow, list);	/* get the FSSpec */
		LDelRow( 1, srcRow.v, list);
		LDoDraw( FALSE, list);
		LAddRow( 1, dstRow.v, list);
		LSetCell( dataPtr, dataLen, dstRow, list);	/* set the label */
		dstRow.h = 1;
		LSetCell( &theSpec, specLen, dstRow, list);	/* set the FSSpec */
		DrawList( list);
		DisposPtr( dataPtr);
		ArrowCursor();
	}
}

