/*
Copyright  1991-1992.  All rights reserved.
	

	You may distribute under the terms of either the GNU General Public
	License or the Artistic License, as specified in the README file.
	
	This file was developed by the FilterTop development team, specifcally 
	this file was written by Chris Priestley & Peter Lewis in early 1992
	See the Modification History for more details

FILE
	filterInterface.h

NAME
	filterInterface.h for ftEngine. Part of the FilterTop project

DESCRIPTION
	This source file contains code for prototypes that handle
	the filter interface.

MODIFICATION HISTORY
	Created by Chris Priestley & Peter Lewis

	dd mmm yy	-	xxx	-	patchxx: description of patch
	25 Mar 92	-	CRP	-	No Changes
	
*/	

/*********************************** filterInterface.h ******************************/

#ifndef	_filterInterface_
#define	_filterInterface_

#define				MAXFILTERS	100

#define	idle					0
#define	waitingForStart			1
#define	active					2


/*************************** Chris's typedefs ************************/

typedef pascal OSErr (**funcHdl)( PBCalltoFilterPtr);   /* handle to pascal function returning OSErr */

typedef struct listEntry
{
	struct listEntry	*before;
	struct listEntry	*after;
} *listPtr, *listHead;

typedef struct				
{							
	long	filterVersion;	
	long	minFTversion;	
	long	curFTversion;	
	long	memRequired;	
	long	IconNumber;
	unsigned long 	configPossible : 1;	
	unsigned long	configRequired : 1;	
	unsigned long	autoConfig : 1;	
	unsigned long	usesStreams : 1;	
	unsigned long	usesFSSpecs : 1;	
	unsigned long	reserved5 : 1;	
	unsigned long	reserved6 : 1;	
	unsigned long	reserved7 : 1;
	long			reserved3;	
	long			reserved4;	
		
} fINFrecord, *fINFrecordPtr;

typedef struct							/* for each filter read in at start up. */
{							
	struct listEntry	listHeader;		/* for linked list unit */
	funcHdl				filterFunc;		/* handle to the filter. Moved hi and locked down when needed */
	fINFrecord			fINFrec;		/* handle to hold fINF resource */
	Handle				filterGlobalsH;	/* not yet used */
	Handle				filterConfigH;	/* handle to hold fCFG resource */
	long				count;
	FSSpec				spec;			/* spec for the filter */
} FilterInfoStruct, *FilterInfoStructPtr;

typedef struct FTDataRecord		/* list entry for a FSSpec/stream */
{
	struct listEntry		listHeader;
	FSSpec					fs;				/* file spec */
	FSSpec					tempFS;			/* file spec for temp files*/
	OSType					creator;		/* file or stream creator */
	OSType					type;			/* file or stream type */
	Handle					buffer;			/* data for stream */
	short					refNum;			/* for an open file */
	short					reading;		
	short					writing;
	Ptr						sourceFilter;	/* for stalling the source filter */
	
}*FTDataPtr, **FTDataHdl;

typedef struct ActiveFilterStruct	/* Info for "active" filters, ie, those being used */
{
	struct listEntry			listHeader;
	long						processNumber;		/* needed for the process window */
	long						filterNumber;
	struct ActiveFilterStruct	*source;			/* pointer to the source ActiveFilterStruct in a pipeLine */
	struct ActiveFilterStruct	*dest;				/* pointer to the dest ActiveFilterStruct in a pipeLine */
	funcHdl						filterFunc;			/* handle to the filter */
	fINFrecord					fINFrec;
	long						state;				/* is it processing or not. 0 = not, 1 = is */
	Boolean						stalled;
	long						stalledStream;
	struct FTDataRecord			*FTdata;			/* handle of FSSpecs to work on */
	PBCallToFilter				pb;					/* pb holding data for the filter's use */
};

typedef struct {				/* this struct is stuffed into hidden cell of process window list */
	long						processNumber;	/* to match the process with the name in the list */
	short						iconNum;		/* which icon should processLDEF display */
	short						boldState;		/* so processLDEF knows whether to bold or plain the name */
}ProcessInfo;

/****************************** protos **********************/

void			FilterInit(void);
void			GetAppDir( void);						/* Grab the dirID of the app directory at startup */
void			AddFilters( ListHandle);				/* load each filter in the Filters Folder */
void			lockFilterHandles( void);				/* move and lock down all the filter handles */
void			unLockFilterHandles( void);				/* unlock all filter handles at exit */
void			AddFilterIcon( ListHandle, short);		/* grab the appropriate icon and stuff it in Filter list */
void			AddLineToPipeList( ListHandle, StringPtr);

/************************** filter interface *************************/

void						HandleIndividualActiveFilter( short);	/* double click on a filter */
void						HandlePipeLine( void);

Boolean						HandleFilterLoop( struct ActiveFilterStruct *);		/* null event, so go thru filter loop*/
OSErr						HandleMessage( struct ActiveFilterStruct *, long);	/* handle all messages except opn and close */
void						ReportError( OSErr, struct ActiveFilterStruct *);

void						SetUpActiveFilterRecord( struct ActiveFilterStruct *, short);
OSErr						HandleOpenFilterRes( short);
struct ActiveFilterStruct	*InstallActiveFilter( void);
OSErr						DeleteInactiveFilter( struct ActiveFilterStruct *);

void						InstallProcessWindowEntry( struct ActiveFilterStruct *, short);
void						DeleteProcessWindowEntry( struct ActiveFilterStruct *);
void						HiliteProcessWindowEntry( struct ActiveFilterStruct *, short);

void						SetupFileArray( struct ActiveFilterStruct *);
struct FTDataRecord 		*InstallNewFTDataListEntry( struct ActiveFilterStruct *);
OSErr						DeleteFTDataList( struct ActiveFilterStruct *);

struct FTDataRecord 		*GetFSSpecFromFTDataList( struct ActiveFilterStruct *);                                   
struct FTDataRecord 		*GetStreamFromFTDataList( struct ActiveFilterStruct *);                                   
OSErr						CheckForInputAvailable( struct ActiveFilterStruct *);

OSErr						CreateOutputFile( struct FTDataRecord *);
short						MakeTempFile( FSSpec *, FSSpec *);
short						SaveTempToOriginal( struct FTDataRecord	*);

listPtr						AddtoTail( listHead *, long);                                            
void 						DeleteItem( listHead *, struct listEntry *);

OSErr						AddDataToHandle(Handle, Ptr, long);
OSErr						GetDataFromHandle(Handle, Ptr, long *);

OSErr 						TouchDir ( short, long );

/**************************************** callbacks *********************************/

void			initCallBackArray( void);	/* put addresses in global cb */

pascal OSErr 	StreamOpenIn(Stream *strm, FSSpec *spec, OSType *creator, OSType *type);
pascal OSErr 	StreamCloseIn(Stream *strm);
pascal OSErr 	GetFSSpec(FSSpec *spec, OSType *creator, OSType *type);
pascal OSErr	SetFSSpec(short path, FSSpec *spec, OSType *creator, OSType *type);
pascal long 	StreamAvaibleInputBytes(Stream strm);
pascal OSErr	StreamReadBytes(Stream strm, void *bufferP, long *noBytes);
pascal OSErr	StreamOpenOut(Stream *strm, short path, FSSpec *spec, OSType creator,OSType type);
pascal OSErr	StreamCloseOut(Stream *strm);
pascal OSErr	StreamWriteBytes(Stream strm, const void *bufferP, long noBytes);
pascal OSErr	StreamWriteString(Stream strm, Str255 s);
pascal OSErr 	StreamReadUpTo(Stream strm, void *bufferP, long *noBytes, char upto, short *gotchar);


#endif