/* 
==========================================================================
     crypt.c (Stream version) Adrian Vansyl 1992
     This version has a configurable key.
     
     This filter works both ways.
     If the file is unencrypted, it encrypts it.
     If the file is encrypted, it unencryts it.
     
==========================================================================
*/

#include "filter.h"

/*************************** Self Config Protos *****************/

short 			cfgSelfInit (PBCallToFilter *);
short 			cfgSelfHandleEvent (PBCallToFilter *);
short 			cfgSelfClose (PBCallToFilter *);

/*************** protos for filter's internal use *********************/

short			ConvertBuffers( char *, char *, long, Handle);
char 			*cStrcpyn(char *,char *, short);

/************************** main *****************************/

pascal OSErr     main (PBCallToFilter *pb)
{
	OSErr		oe;
	OSType		creator, ftype;
	FSSpec 		fs;
	long		count;
	char		inBuffer[512], outBuffer[512];

	oe = noErr;	
	switch( pb->serviceRequested )
	{
		case msgOpenFilter:
			break;

		case msgCloseFilter:
			break;

		case msgStartFiltering:		
			oe = pb->cb->StreamOpenIn( &pb->streamIn, &fs, &creator, &ftype);
			if( oe == noErr)
			{
				oe = pb->cb->StreamOpenOut( &pb->streamOut, 1, &fs, creator, ftype);
				oe = inProgress;
			}
			else pb->streamIn = 0L;			

			break;

		case msgFinishFiltering:
			break;					

		case msgFilterAWhile:
			
			if( pb->streamIn == 0L)
			{ 
				oe = -1;
				break;
			}
			
			count = 512;
			if(( oe = pb->cb->StreamReadBytes( pb->streamIn, inBuffer, &count)) == noErr)
			{
				ConvertBuffers( inBuffer, outBuffer, count, pb->filterConfigH);				
				oe = pb->cb->StreamWriteBytes( pb->streamOut, outBuffer, count);
				oe = inProgress;
			}
			else
			{
				oe = pb->cb->StreamCloseIn( &pb->streamIn);
				oe = pb->cb->StreamCloseOut( &pb->streamOut);
				oe = noErr;								
			}

			break;

		case msgAbortFiltering:
			break;

		case msgInitOptions:
			cfgSelfInit(pb);
			break;
		
		case msgDialogEvent:
			pb->var5 = cfgSelfHandleEvent (pb);
			break;

		case msgCloseOptions:
			cfgSelfClose(pb);
			break;

		case msgRankThis:
			break;

		otherwise:
			oe = errUnknownMessage;
			break;
	}
	return oe;
}


/****************************** ConvertBuffers ****************************/

short		ConvertBuffers( inBuf, outBuf, count, configH)
char		*inBuf, *outBuf;	
long		count;
Handle		configH;
{
	short		i, keyoffset, keylength;
	char		key[60];
	StringPtr	userString;
	
	userString = (StringPtr)*configH;
	
	cStrcpyn( key, (char *)&userString[1], userString[0]+1); /* this is the key used to crypt */
	
	keylength = userString[0];
	
	for( i = 0; i < count; ++i)					/* loop thru all chars in inBuf */
	{
		keyoffset = i % keylength;  			/* cycle through all the letters in key */
		outBuf[i] = inBuf[i] ^ key[keyoffset];  /* do an EOR with each char */
	}
	return noErr;
}			


/*********************************** cStrcpy ***************************/
/* Copy C String, returns destination string */

char     *cStrcpyn(s,t,n)
char    *s, *t;
short 	n;
{
	short   i;

	for (i = 0; i < n && t[i] != '\0'; ++i)  s[i] = t[i];
	s[i] = '\0';
	return (s);
}





/********************************** Self Configuration ******************************/
/********************************** Self Configuration ******************************/
/********************************** Self Configuration ******************************/

/*--------- Typedefs -----------*/

typedef short state;
#define	cfgStNone	0
#define cfgStOk		1
#define cfgStCancel	2
#define cfgStSave	3

/****************************** cfgSelfInit *********************************/

short 			cfgSelfInit (PBCallToFilter *pb)
{
	short	iType;
	Handle	iHandle;
	Rect	iRect;
	
	SetPort(pb->dlg);
	GetDItem(pb->dlg, 4, &iType, &iHandle, &iRect);
	SetIText( iHandle, (StringPtr)*pb->filterConfigH);
	SelIText( pb->dlg, 4, 0, 32767);
	ShowWindow ( pb->dlg );
}

/****************************** cfgSelfHandleEvent *********************************/

short cfgSelfHandleEvent (PBCallToFilter *pb)
{
short			itemHit,iType;
ControlHandle	iHandle;
DialogPtr		whichD, theD;
EventRecord		*evt;
char 			theChar;
Rect			iRect;
Point			where;
Cursor			editCursor;
CursHandle		curH;


	theD = pb->dlg;
	evt = pb->event;
	where = evt->where;
	GlobalToLocal(&where);

	switch (evt->what) 		/* keyboard support */
	{
		case keyDown:
			theChar = evt->message & charCodeMask;
			if ((theChar==0x03) || (theChar==0x0D)) 
			{
				long	d;
				GetDItem ( theD, 1, &iType, (Handle *)&iHandle, &iRect );
				HiliteControl ( iHandle, 1 );
				Delay(2,&d);
				HiliteControl ( iHandle, 0 );
				return cfgStOk;
			}
		break;
		
		case updateEvt:
			GetDItem(pb->dlg, 1, &iType, (Handle *)&iHandle, &iRect);
			PenSize(3,3);
			InsetRect(&iRect, -4, -4);
			FrameRoundRect(&iRect, 16, 16);
	}
	
	if( FindDItem ( theD, where ) == 3 )	/* Cursor Control */
	{
		curH = GetCursor( iBeamCursor);
		editCursor = **curH;
		SetCursor( &editCursor);
	}
	else
		InitCursor();

	if ( DialogSelect ( evt, &whichD, &itemHit ) ) 
		if (whichD == theD)
		{
			switch (itemHit)
			{
				case 1:
					return cfgStOk;
				break;

				case 2:
					return cfgStCancel;
				break;

				case 3:
					return cfgStSave;
				break;
				
			}
		}

	return cfgStNone;

}

/****************************** cfgSelfClose *********************************/

short 			cfgSelfClose (PBCallToFilter *pb)
{
	short	iType, i;
	Handle	iHandle;
	Rect	iRect;
	
	if(pb->var5 == cfgStOk || pb->var5 == cfgStSave)
	{
		GetDItem(pb->dlg, 4, &iType, &iHandle, &iRect);
		
		if( GetHandleSize( pb->filterConfigH) != sizeof(Str255) )
			SetHandleSize( pb->filterConfigH, sizeof(Str255));
		
		HLock( (Handle)pb->filterConfigH);
		GetIText( iHandle, (StringPtr)*pb->filterConfigH);
		HUnlock( (Handle)pb->filterConfigH);
	}
}





